from django.contrib.gis.utils.layermapping import LayerMapError as LayerMapError
from django.contrib.gis.utils.layermapping import LayerMapping as LayerMapping
from django.contrib.gis.utils.ogrinfo import ogrinfo as ogrinfo
from django.contrib.gis.utils.ogrinspect import mapping as mapping
from django.contrib.gis.utils.ogrinspect import ogrinspect as ogrinspect
from django.contrib.gis.utils.srs import add_srs_entry as add_srs_entry



from collections.abc import Container
from typing import Any

from django.core.management.base import BaseCommand
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.migrations.operations.base import Operation

class Command(BaseCommand):
    verbosity: int
    interactive: bool
    start: float
    def migration_progress_callback(self, action: str, migration: Any | None = ..., fake: bool = ...) -> None: ...
    def sync_apps(self, connection: BaseDatabaseWrapper, app_labels: Container[str]) -> None: ...
    @staticmethod
    def describe_operation(operation: Operation, backwards: bool) -> tuple[str, bool]: ...



from typing import Any, List, Tuple, Union

from django.contrib.admin.options import BaseModelAdmin
from django.http.request import HttpRequest


class HiddenFieldsMixin(BaseModelAdmin):
    hidden_fields: Union[List[str], Tuple[str]] = []

    def get_fields(self, request: HttpRequest, obj: Union[Any, None] = ...):
        """
        Exclude all fields that may hold sensitive data
        """

        fields = super().get_fields(request, obj)
        fields_list = list(fields)

        if obj and self.hidden_fields:
            for field in self.hidden_fields:
                fields_list.remove(field)

        return tuple(fields_list)



import sys

# actually csv.Dialect is a different class to _csv.Dialect at runtime, but for typing purposes, they're identical
from _csv import (
    QUOTE_ALL as QUOTE_ALL,
    QUOTE_MINIMAL as QUOTE_MINIMAL,
    QUOTE_NONE as QUOTE_NONE,
    QUOTE_NONNUMERIC as QUOTE_NONNUMERIC,
    Dialect as Dialect,
    Error as Error,
    __version__ as __version__,
    _DialectLike,
    _QuotingType,
    _reader,
    _writer,
    field_size_limit as field_size_limit,
    get_dialect as get_dialect,
    list_dialects as list_dialects,
    reader as reader,
    register_dialect as register_dialect,
    unregister_dialect as unregister_dialect,
    writer as writer,
)

if sys.version_info >= (3, 12):
    from _csv import QUOTE_NOTNULL as QUOTE_NOTNULL, QUOTE_STRINGS as QUOTE_STRINGS
from _typeshed import SupportsWrite
from collections.abc import Collection, Iterable, Iterator, Mapping, Sequence
from typing import Any, Generic, TypeVar, overload
from typing_extensions import Literal, Self

if sys.version_info >= (3, 8):
    from builtins import dict as _DictReadMapping
else:
    from collections import OrderedDict as _DictReadMapping

if sys.version_info >= (3, 12):
    from types import GenericAlias

__all__ = [
    "QUOTE_MINIMAL",
    "QUOTE_ALL",
    "QUOTE_NONNUMERIC",
    "QUOTE_NONE",
    "Error",
    "Dialect",
    "__doc__",
    "excel",
    "excel_tab",
    "field_size_limit",
    "reader",
    "writer",
    "register_dialect",
    "get_dialect",
    "list_dialects",
    "Sniffer",
    "unregister_dialect",
    "__version__",
    "DictReader",
    "DictWriter",
    "unix_dialect",
]
if sys.version_info >= (3, 12):
    __all__ += ["QUOTE_STRINGS", "QUOTE_NOTNULL"]

_T = TypeVar("_T")

class excel(Dialect): ...
class excel_tab(excel): ...
class unix_dialect(Dialect): ...

class DictReader(Iterator[_DictReadMapping[_T | Any, str | Any]], Generic[_T]):
    fieldnames: Sequence[_T] | None
    restkey: _T | None
    restval: str | Any | None
    reader: _reader
    dialect: _DialectLike
    line_num: int
    @overload
    def __init__(
        self,
        f: Iterable[str],
        fieldnames: Sequence[_T],
        restkey: _T | None = None,
        restval: str | Any | None = None,
        dialect: _DialectLike = "excel",
        *,
        delimiter: str = ",",
        quotechar: str | None = '"',
        escapechar: str | None = None,
        doublequote: bool = True,
        skipinitialspace: bool = False,
        lineterminator: str = "\r\n",
        quoting: _QuotingType = 0,
        strict: bool = False,
    ) -> None: ...
    @overload
    def __init__(
        self: DictReader[str],
        f: Iterable[str],
        fieldnames: Sequence[str] | None = None,
        restkey: str | None = None,
        restval: str | None = None,
        dialect: _DialectLike = "excel",
        *,
        delimiter: str = ",",
        quotechar: str | None = '"',
        escapechar: str | None = None,
        doublequote: bool = True,
        skipinitialspace: bool = False,
        lineterminator: str = "\r\n",
        quoting: _QuotingType = 0,
        strict: bool = False,
    ) -> None: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _DictReadMapping[_T | Any, str | Any]: ...
    if sys.version_info >= (3, 12):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class DictWriter(Generic[_T]):
    fieldnames: Collection[_T]
    restval: Any | None
    extrasaction: Literal["raise", "ignore"]
    writer: _writer
    def __init__(
        self,
        f: SupportsWrite[str],
        fieldnames: Collection[_T],
        restval: Any | None = "",
        extrasaction: Literal["raise", "ignore"] = "raise",
        dialect: _DialectLike = "excel",
        *,
        delimiter: str = ",",
        quotechar: str | None = '"',
        escapechar: str | None = None,
        doublequote: bool = True,
        skipinitialspace: bool = False,
        lineterminator: str = "\r\n",
        quoting: _QuotingType = 0,
        strict: bool = False,
    ) -> None: ...
    if sys.version_info >= (3, 8):
        def writeheader(self) -> Any: ...
    else:
        def writeheader(self) -> None: ...

    def writerow(self, rowdict: Mapping[_T, Any]) -> Any: ...
    def writerows(self, rowdicts: Iterable[Mapping[_T, Any]]) -> None: ...
    if sys.version_info >= (3, 12):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class Sniffer:
    preferred: list[str]
    def sniff(self, sample: str, delimiters: str | None = None) -> type[Dialect]: ...
    def has_header(self, sample: str) -> bool: ...



from typing import Callable, Optional

from django.db.models import Model
from pydantic import BaseModel, ConfigDict, Field

LoginFunction = Callable[[Model, str], str]


class PermissionTestExpectation(BaseModel):
    user: Model = Field(
        description="Should be AbstractBaseUser but this lead to a circular import. "
    )
    password: str = Field(description="Password that is used for the user to login")
    get_list_status_code: Optional[int] = None
    get_list_return_number: Optional[int] = None
    get_detail_status_code: Optional[int] = None
    create_status_code: Optional[int] = None
    patch_status_code: Optional[int] = None
    put_status_code: Optional[int] = None
    delete_status_code: Optional[int] = None

    model_config = ConfigDict(arbitrary_types_allowed=True)



from typing import Any

from django.db.models.query_utils import DeferredAttribute

class SpatialProxy(DeferredAttribute):
    def __init__(self, klass: Any, field: Any, load_func: Any | None = ...) -> None: ...
    def __get__(self, instance: Any, cls: Any | None = ...) -> Any: ...
    def __set__(self, instance: Any, value: Any) -> Any: ...



from _typeshed import StrPath
from typing import ClassVar, Literal

from .. import fixer_base
from ..pytree import Node

class FixUnicode(fixer_base.BaseFix):
    BM_compatible: ClassVar[Literal[True]]
    PATTERN: ClassVar[Literal["STRING | 'unicode' | 'unichr'"]]  # type: ignore[name-defined]  # Name "STRING" is not defined
    unicode_literals: bool
    def start_tree(self, tree: Node, filename: StrPath) -> None: ...
    def transform(self, node, results): ...



from typing import Any

from ..cmd import Command

def show_formats() -> None: ...

class bdist(Command):
    description: str
    user_options: Any
    boolean_options: Any
    help_options: Any
    no_format_option: Any
    default_format: Any
    format_commands: Any
    format_command: Any
    bdist_base: Any
    plat_name: Any
    formats: Any
    dist_dir: Any
    skip_build: int
    group: Any
    owner: Any
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...



from collections.abc import Iterator
from typing import Any

from django.contrib.gis.geos.geometry import GEOSGeometry
from django.contrib.gis.geos.linestring import LinearRing

class Polygon(GEOSGeometry):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __iter__(self) -> Iterator[LinearRing]: ...
    def __len__(self) -> int: ...
    @classmethod
    def from_bbox(cls, bbox: tuple[float, float, float, float]) -> Polygon: ...
    @property
    def num_interior_rings(self) -> int: ...
    exterior_ring: LinearRing
    shell: LinearRing
    @property
    def coords(self) -> tuple[tuple[tuple[float, ...]]]: ...
    @property
    def tuple(self) -> tuple[tuple[tuple[float, ...]]]: ...



from typing import Any

from django.db import models
from django.http.request import HttpRequest

SITE_CACHE: Any

class SiteManager(models.Manager[Site]):
    def get_current(self, request: HttpRequest | None = ...) -> Site: ...
    def clear_cache(self) -> None: ...
    def get_by_natural_key(self, domain: str) -> Site: ...

class Site(models.Model):
    objects: SiteManager

    domain = models.CharField(max_length=100)
    name = models.CharField(max_length=50)
    def natural_key(self) -> tuple[str]: ...

def clear_site_cache(sender: type[Site], **kwargs: Any) -> None: ...



import datetime
from collections.abc import Collection, Iterable, Iterator, MutableMapping, Sequence
from typing import Any, Generic, NoReturn, TypeVar, overload

from _typeshed import Self
from django.db.models import Combinable
from django.db.models.base import Model
from django.db.models.query import QuerySet, RawQuerySet

from django_stubs_ext import ValuesQuerySet

_T = TypeVar("_T", bound=Model, covariant=True)

class BaseManager(Generic[_T]):
    creation_counter: int
    auto_created: bool
    use_in_migrations: bool
    name: str
    model: type[_T]
    _db: str | None
    def __init__(self) -> None: ...
    def deconstruct(
        self,
    ) -> tuple[bool, str | None, str | None, tuple[Any, ...] | None, dict[str, Any] | None]: ...
    def check(self, **kwargs: Any) -> list[Any]: ...
    @classmethod
    def from_queryset(
        cls: type[Self], queryset_class: type[QuerySet[_T]], class_name: str | None = ...
    ) -> type[Self]: ...
    @classmethod
    def _get_queryset_methods(cls, queryset_class: type) -> dict[str, Any]: ...
    def contribute_to_class(self, cls: type[Model], name: str) -> None: ...
    def db_manager(self: Self, using: str | None = ..., hints: dict[str, Model] | None = ...) -> Self: ...
    @property
    def db(self) -> str: ...
    def get_queryset(self) -> QuerySet[_T]: ...
    # NOTE: The following methods are in common with QuerySet, but note that the use of QuerySet as a return type
    # rather than a self-type (_QS), since Manager's QuerySet-like methods return QuerySets and not Managers.
    def iterator(self, chunk_size: int = ...) -> Iterator[_T]: ...
    async def aiterator(self, chunk_size: int = ...) -> Iterator[_T]: ...
    def aggregate(self, *args: Any, **kwargs: Any) -> dict[str, Any]: ...
    async def aaggregate(self, *args: Any, **kwargs: Any) -> dict[str, Any]: ...
    def get(self, *args: Any, **kwargs: Any) -> _T: ...
    async def aget(self, *args: Any, **kwargs: Any) -> _T: ...
    def create(self, **kwargs: Any) -> _T: ...
    async def acreate(self, **kwargs: Any) -> _T: ...
    def bulk_create(
        self,
        objs: Iterable[_T],
        batch_size: int | None = ...,
        ignore_conflicts: bool = ...,
        update_conflicts: bool = ...,
        update_fields: Collection[str] | None = ...,
        unique_fields: Collection[str] | None = ...,
    ) -> list[_T]: ...
    async def abulk_create(
        self,
        objs: Iterable[_T],
        batch_size: int | None = ...,
        ignore_conflicts: bool = ...,
        update_conflicts: bool = ...,
        update_fields: Collection[str] | None = ...,
        unique_fields: Collection[str] | None = ...,
    ) -> list[_T]: ...
    def bulk_update(self, objs: Iterable[_T], fields: Sequence[str], batch_size: int | None = ...) -> int: ...
    async def abulk_update(self, objs: Iterable[_T], fields: Sequence[str], batch_size: int | None = ...) -> int: ...
    def get_or_create(self, defaults: MutableMapping[str, Any] | None = ..., **kwargs: Any) -> tuple[_T, bool]: ...
    async def aget_or_create(
        self, defaults: MutableMapping[str, Any] | None = ..., **kwargs: Any
    ) -> tuple[_T, bool]: ...
    def update_or_create(self, defaults: MutableMapping[str, Any] | None = ..., **kwargs: Any) -> tuple[_T, bool]: ...
    async def aupdate_or_create(
        self, defaults: MutableMapping[str, Any] | None = ..., **kwargs: Any
    ) -> tuple[_T, bool]: ...
    def earliest(self, *fields: Any, field_name: Any | None = ...) -> _T: ...
    async def aearliest(self, *fields: Any, field_name: Any | None = ...) -> _T: ...
    def latest(self, *fields: Any, field_name: Any | None = ...) -> _T: ...
    async def alatest(self, *fields: Any, field_name: Any | None = ...) -> _T: ...
    def first(self) -> _T | None: ...
    async def afirst(self) -> _T | None: ...
    def last(self) -> _T | None: ...
    async def alast(self) -> _T | None: ...
    def in_bulk(self, id_list: Iterable[Any] = ..., *, field_name: str = ...) -> dict[Any, _T]: ...
    async def ain_bulk(self, id_list: Iterable[Any] = ..., *, field_name: str = ...) -> dict[Any, _T]: ...
    def delete(self) -> tuple[int, dict[str, int]]: ...
    async def adelete(self) -> tuple[int, dict[str, int]]: ...
    def update(self, **kwargs: Any) -> int: ...
    async def aupdate(self, **kwargs: Any) -> int: ...
    def exists(self) -> bool: ...
    async def aexists(self) -> bool: ...
    def explain(self, *, format: Any | None = ..., **options: Any) -> str: ...
    async def aexplain(self, *, format: Any | None = ..., **options: Any) -> str: ...
    def contains(self, objs: Model) -> bool: ...
    async def acontains(self, objs: Model) -> bool: ...
    def raw(
        self,
        raw_query: str,
        params: Any = ...,
        translations: dict[str, str] | None = ...,
        using: str | None = ...,
    ) -> RawQuerySet: ...
    # The type of values may be overridden to be more specific in the mypy plugin, depending on the fields param
    def values(self, *fields: str | Combinable, **expressions: Any) -> ValuesQuerySet[_T, dict[str, Any]]: ...
    # The type of values_list may be overridden to be more specific in the mypy plugin, depending on the fields param
    def values_list(
        self, *fields: str | Combinable, flat: bool = ..., named: bool = ...
    ) -> ValuesQuerySet[_T, Any]: ...
    def dates(self, field_name: str, kind: str, order: str = ...) -> ValuesQuerySet[_T, datetime.date]: ...
    def datetimes(
        self, field_name: str, kind: str, order: str = ..., tzinfo: datetime.tzinfo | None = ...
    ) -> ValuesQuerySet[_T, datetime.datetime]: ...
    def none(self) -> QuerySet[_T]: ...
    def all(self) -> QuerySet[_T]: ...
    def filter(self, *args: Any, **kwargs: Any) -> QuerySet[_T]: ...
    def exclude(self, *args: Any, **kwargs: Any) -> QuerySet[_T]: ...
    def complex_filter(self, filter_obj: Any) -> QuerySet[_T]: ...
    def count(self) -> int: ...
    async def acount(self) -> int: ...
    def union(self, *other_qs: Any, all: bool = ...) -> QuerySet[_T]: ...
    def intersection(self, *other_qs: Any) -> QuerySet[_T]: ...
    def difference(self, *other_qs: Any) -> QuerySet[_T]: ...
    def select_for_update(
        self, nowait: bool = ..., skip_locked: bool = ..., of: Sequence[str] = ..., no_key: bool = ...
    ) -> QuerySet[_T]: ...
    def select_related(self, *fields: Any) -> QuerySet[_T]: ...
    def prefetch_related(self, *lookups: Any) -> QuerySet[_T]: ...
    def annotate(self, *args: Any, **kwargs: Any) -> QuerySet[_T]: ...
    def alias(self, *args: Any, **kwargs: Any) -> QuerySet[_T]: ...
    def order_by(self, *field_names: Any) -> QuerySet[_T]: ...
    def distinct(self, *field_names: Any) -> QuerySet[_T]: ...
    # extra() return type won't be supported any time soon
    def extra(
        self,
        select: dict[str, Any] | None = ...,
        where: list[str] | None = ...,
        params: list[Any] | None = ...,
        tables: list[str] | None = ...,
        order_by: Sequence[str] | None = ...,
        select_params: Sequence[Any] | None = ...,
    ) -> QuerySet[Any]: ...
    def reverse(self) -> QuerySet[_T]: ...
    def defer(self, *fields: Any) -> QuerySet[_T]: ...
    def only(self, *fields: Any) -> QuerySet[_T]: ...
    def using(self, alias: str | None) -> QuerySet[_T]: ...
    @property
    def ordered(self) -> bool: ...

class Manager(BaseManager[_T]): ...

# Fake to make ManyToMany work
class RelatedManager(Manager[_T]):
    related_val: tuple[int, ...]
    def add(self, *objs: _T | int, bulk: bool = ...) -> None: ...
    def remove(self, *objs: _T | int, bulk: bool = ...) -> None: ...
    def set(self, objs: QuerySet[_T] | Iterable[_T | int], *, bulk: bool = ..., clear: bool = ...) -> None: ...
    def clear(self) -> None: ...
    def __call__(self, *, manager: str) -> RelatedManager[_T]: ...

class ManagerDescriptor:
    manager: BaseManager
    def __init__(self, manager: BaseManager) -> None: ...
    @overload
    def __get__(self, instance: None, cls: type[Model] | None = ...) -> BaseManager: ...
    @overload
    def __get__(self, instance: Model, cls: type[Model] | None = ...) -> NoReturn: ...

class EmptyManager(Manager[_T]):
    def __init__(self, model: type[_T]) -> None: ...



from typing import Any

class WKTAdapter:
    wkt: Any
    srid: Any
    def __init__(self, geom: Any) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...



from typing import ClassVar
from typing_extensions import Literal

from .. import fixer_base

class FixZip(fixer_base.ConditionalFix):
    BM_compatible: ClassVar[Literal[True]]
    PATTERN: ClassVar[str]
    skip_on: ClassVar[Literal["future_builtins.zip"]]
    def transform(self, node, results): ...



from abc import ABCMeta, abstractmethod
from collections.abc import Callable
from email.errors import MessageDefect
from email.header import Header
from email.message import Message
from typing import Any
from typing_extensions import Self

class _PolicyBase:
    def __add__(self, other: Any) -> Self: ...
    def clone(self, **kw: Any) -> Self: ...

class Policy(_PolicyBase, metaclass=ABCMeta):
    max_line_length: int | None
    linesep: str
    cte_type: str
    raise_on_defect: bool
    mangle_from_: bool
    message_factory: Callable[[Policy], Message] | None
    def __init__(
        self,
        *,
        max_line_length: int | None = 78,
        linesep: str = "\n",
        cte_type: str = "8bit",
        raise_on_defect: bool = False,
        mangle_from_: bool = False,
        message_factory: Callable[[Policy], Message] | None = None,
    ) -> None: ...
    def handle_defect(self, obj: Message, defect: MessageDefect) -> None: ...
    def register_defect(self, obj: Message, defect: MessageDefect) -> None: ...
    def header_max_count(self, name: str) -> int | None: ...
    @abstractmethod
    def header_source_parse(self, sourcelines: list[str]) -> tuple[str, str]: ...
    @abstractmethod
    def header_store_parse(self, name: str, value: str) -> tuple[str, str]: ...
    @abstractmethod
    def header_fetch_parse(self, name: str, value: str) -> str: ...
    @abstractmethod
    def fold(self, name: str, value: str) -> str: ...
    @abstractmethod
    def fold_binary(self, name: str, value: str) -> bytes: ...

class Compat32(Policy):
    def header_source_parse(self, sourcelines: list[str]) -> tuple[str, str]: ...
    def header_store_parse(self, name: str, value: str) -> tuple[str, str]: ...
    def header_fetch_parse(self, name: str, value: str) -> str | Header: ...  # type: ignore[override]
    def fold(self, name: str, value: str) -> str: ...
    def fold_binary(self, name: str, value: str) -> bytes: ...

compat32: Compat32



import sys
from _typeshed import AnyStr_co, StrOrBytesPath
from collections.abc import Callable, Iterable, Iterator
from types import TracebackType
from typing import IO, Any, AnyStr, Literal, Protocol, overload
from typing_extensions import Self, TypeAlias

if sys.version_info >= (3, 9):
    from types import GenericAlias

__all__ = [
    "input",
    "close",
    "nextfile",
    "filename",
    "lineno",
    "filelineno",
    "fileno",
    "isfirstline",
    "isstdin",
    "FileInput",
    "hook_compressed",
    "hook_encoded",
]

if sys.version_info >= (3, 11):
    _TextMode: TypeAlias = Literal["r"]
else:
    _TextMode: TypeAlias = Literal["r", "rU", "U"]

class _HasReadlineAndFileno(Protocol[AnyStr_co]):
    def readline(self) -> AnyStr_co: ...
    def fileno(self) -> int: ...

if sys.version_info >= (3, 10):
    # encoding and errors are added
    @overload
    def input(
        files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
        inplace: bool = False,
        backup: str = "",
        *,
        mode: _TextMode = "r",
        openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[str]] | None = None,
        encoding: str | None = None,
        errors: str | None = None,
    ) -> FileInput[str]: ...
    @overload
    def input(
        files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
        inplace: bool = False,
        backup: str = "",
        *,
        mode: Literal["rb"],
        openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[bytes]] | None = None,
        encoding: None = None,
        errors: None = None,
    ) -> FileInput[bytes]: ...
    @overload
    def input(
        files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
        inplace: bool = False,
        backup: str = "",
        *,
        mode: str,
        openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[Any]] | None = None,
        encoding: str | None = None,
        errors: str | None = None,
    ) -> FileInput[Any]: ...

else:
    # bufsize is dropped and mode and openhook become keyword-only
    @overload
    def input(
        files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
        inplace: bool = False,
        backup: str = "",
        *,
        mode: _TextMode = "r",
        openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[str]] | None = None,
    ) -> FileInput[str]: ...
    @overload
    def input(
        files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
        inplace: bool = False,
        backup: str = "",
        *,
        mode: Literal["rb"],
        openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[bytes]] | None = None,
    ) -> FileInput[bytes]: ...
    @overload
    def input(
        files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
        inplace: bool = False,
        backup: str = "",
        *,
        mode: str,
        openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[Any]] | None = None,
    ) -> FileInput[Any]: ...

def close() -> None: ...
def nextfile() -> None: ...
def filename() -> str: ...
def lineno() -> int: ...
def filelineno() -> int: ...
def fileno() -> int: ...
def isfirstline() -> bool: ...
def isstdin() -> bool: ...

class FileInput(Iterator[AnyStr]):
    if sys.version_info >= (3, 10):
        # encoding and errors are added
        @overload
        def __init__(
            self: FileInput[str],
            files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
            inplace: bool = False,
            backup: str = "",
            *,
            mode: _TextMode = "r",
            openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[str]] | None = None,
            encoding: str | None = None,
            errors: str | None = None,
        ) -> None: ...
        @overload
        def __init__(
            self: FileInput[bytes],
            files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
            inplace: bool = False,
            backup: str = "",
            *,
            mode: Literal["rb"],
            openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[bytes]] | None = None,
            encoding: None = None,
            errors: None = None,
        ) -> None: ...
        @overload
        def __init__(
            self: FileInput[Any],
            files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
            inplace: bool = False,
            backup: str = "",
            *,
            mode: str,
            openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[Any]] | None = None,
            encoding: str | None = None,
            errors: str | None = None,
        ) -> None: ...

    else:
        # bufsize is dropped and mode and openhook become keyword-only
        @overload
        def __init__(
            self: FileInput[str],
            files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
            inplace: bool = False,
            backup: str = "",
            *,
            mode: _TextMode = "r",
            openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[str]] | None = None,
        ) -> None: ...
        @overload
        def __init__(
            self: FileInput[bytes],
            files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
            inplace: bool = False,
            backup: str = "",
            *,
            mode: Literal["rb"],
            openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[bytes]] | None = None,
        ) -> None: ...
        @overload
        def __init__(
            self: FileInput[Any],
            files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
            inplace: bool = False,
            backup: str = "",
            *,
            mode: str,
            openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[Any]] | None = None,
        ) -> None: ...

    def __del__(self) -> None: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> AnyStr: ...
    if sys.version_info < (3, 11):
        def __getitem__(self, i: int) -> AnyStr: ...

    def nextfile(self) -> None: ...
    def readline(self) -> AnyStr: ...
    def filename(self) -> str: ...
    def lineno(self) -> int: ...
    def filelineno(self) -> int: ...
    def fileno(self) -> int: ...
    def isfirstline(self) -> bool: ...
    def isstdin(self) -> bool: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

if sys.version_info >= (3, 10):
    def hook_compressed(
        filename: StrOrBytesPath, mode: str, *, encoding: str | None = None, errors: str | None = None
    ) -> IO[Any]: ...

else:
    def hook_compressed(filename: StrOrBytesPath, mode: str) -> IO[Any]: ...

def hook_encoded(encoding: str, errors: str | None = None) -> Callable[[StrOrBytesPath, str], IO[Any]]: ...



import sys
from _typeshed import StrOrBytesPath
from collections.abc import Callable, Sequence
from typing import SupportsIndex

if sys.platform != "win32":
    def cloexec_pipe() -> tuple[int, int]: ...
    def fork_exec(
        args: Sequence[StrOrBytesPath] | None,
        executable_list: Sequence[bytes],
        close_fds: bool,
        pass_fds: tuple[int, ...],
        cwd: str,
        env: Sequence[bytes] | None,
        p2cread: int,
        p2cwrite: int,
        c2pread: int,
        c2pwrite: int,
        errread: int,
        errwrite: int,
        errpipe_read: int,
        errpipe_write: int,
        restore_signals: int,
        call_setsid: int,
        pgid_to_set: int,
        gid: SupportsIndex | None,
        extra_groups: list[int] | None,
        uid: SupportsIndex | None,
        child_umask: int,
        preexec_fn: Callable[[], None],
        allow_vfork: bool,
        /,
    ) -> int: ...



import json
from os import path
from typing import List

from nwon_baseline.file_helper import read_file_content


def read_fixture_file(fixture_path: str) -> List[dict]:
    if not path.isfile(fixture_path):
        return []

    fixture_json = read_file_content(fixture_path)

    if fixture_json is None:
        return []
    else:
        return json.loads(fixture_json)


__all__ = [
    "read_fixture_file",
]



import datetime
from collections.abc import Callable, Iterable, Iterator, Sequence
from decimal import Decimal
from typing import Any, TypeVar

from _typeshed import Self
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import Q
from django.db.models.fields import Field
from django.db.models.lookups import Lookup, Transform
from django.db.models.query import QuerySet
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType
from django.db.models.sql.query import Query
from typing_extensions import Literal, TypeAlias

class SQLiteNumericMixin:
    def as_sqlite(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

_Numeric: TypeAlias = float | Decimal

class Combinable:
    ADD: str
    SUB: str
    MUL: str
    DIV: str
    POW: str
    MOD: str
    BITAND: str
    BITOR: str
    BITLEFTSHIFT: str
    BITRIGHTSHIFT: str
    BITXOR: str
    def __neg__(self) -> CombinedExpression: ...
    def __add__(self, other: datetime.timedelta | Combinable | _Numeric | str | None) -> CombinedExpression: ...
    def __sub__(self, other: datetime.timedelta | Combinable | _Numeric) -> CombinedExpression: ...
    def __mul__(self, other: datetime.timedelta | Combinable | _Numeric) -> CombinedExpression: ...
    def __truediv__(self, other: Combinable | _Numeric) -> CombinedExpression: ...
    def __mod__(self, other: int | Combinable) -> CombinedExpression: ...
    def __pow__(self, other: _Numeric | Combinable) -> CombinedExpression: ...
    def __and__(self, other: Combinable | Q) -> Q: ...
    def bitand(self, other: int) -> CombinedExpression: ...
    def bitleftshift(self, other: int) -> CombinedExpression: ...
    def bitrightshift(self, other: int) -> CombinedExpression: ...
    def bitxor(self, other: int) -> CombinedExpression: ...
    def __or__(self, other: Combinable | Q) -> Q: ...
    def bitor(self, other: int) -> CombinedExpression: ...
    def __radd__(self, other: datetime.datetime | _Numeric | Combinable | None) -> CombinedExpression: ...
    def __rsub__(self, other: _Numeric | Combinable) -> CombinedExpression: ...
    def __rmul__(self, other: _Numeric | Combinable) -> CombinedExpression: ...
    def __rtruediv__(self, other: _Numeric | Combinable) -> CombinedExpression: ...
    def __rmod__(self, other: int | Combinable) -> CombinedExpression: ...
    def __rpow__(self, other: _Numeric | Combinable) -> CombinedExpression: ...
    def __rand__(self, other: Any) -> Combinable: ...
    def __ror__(self, other: Any) -> Combinable: ...

class BaseExpression:
    is_summary: bool
    filterable: bool
    window_compatible: bool
    def __init__(self, output_field: Field | None = ...) -> None: ...
    def get_db_converters(self, connection: BaseDatabaseWrapper) -> list[Callable]: ...
    def get_source_expressions(self) -> list[Any]: ...
    def set_source_expressions(self, exprs: Sequence[Combinable]) -> None: ...
    @property
    def contains_aggregate(self) -> bool: ...
    @property
    def contains_over_clause(self) -> bool: ...
    @property
    def contains_column_references(self) -> bool: ...
    def resolve_expression(
        self: Self,
        query: Any = ...,
        allow_joins: bool = ...,
        reuse: set[str] | None = ...,
        summarize: bool = ...,
        for_save: bool = ...,
    ) -> Self: ...
    @property
    def conditional(self) -> bool: ...
    @property
    def field(self) -> Field: ...
    @property
    def output_field(self) -> Field: ...
    @property
    def convert_value(self) -> Callable: ...
    def get_lookup(self, lookup: str) -> type[Lookup] | None: ...
    def get_transform(self, name: str) -> type[Transform] | None: ...
    def relabeled_clone(self: Self, change_map: dict[str | None, str]) -> Self: ...
    def copy(self: Self) -> Self: ...
    def get_group_by_cols(self: Self, alias: str | None = ...) -> list[Self]: ...
    def get_source_fields(self) -> list[Field | None]: ...
    def asc(
        self,
        *,
        descending: bool = ...,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> OrderBy: ...
    def desc(
        self,
        *,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> OrderBy: ...
    def reverse_ordering(self) -> BaseExpression: ...
    def flatten(self) -> Iterator[BaseExpression]: ...
    def as_sql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    def deconstruct(self) -> Any: ...  # fake

class Expression(BaseExpression, Combinable): ...

class CombinedExpression(SQLiteNumericMixin, Expression):
    connector: str
    lhs: Combinable
    rhs: Combinable
    def __init__(self, lhs: Combinable, connector: str, rhs: Combinable, output_field: Field | None = ...) -> None: ...

class DurationExpression(CombinedExpression):
    def compile(self, side: Combinable, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...

class TemporalSubtraction(CombinedExpression):
    def __init__(self, lhs: Combinable, rhs: Combinable) -> None: ...

class F(Combinable):
    name: str
    def __init__(self, name: str) -> None: ...
    def resolve_expression(
        self,
        query: Any = ...,
        allow_joins: bool = ...,
        reuse: set[str] | None = ...,
        summarize: bool = ...,
        for_save: bool = ...,
    ) -> F: ...
    def asc(
        self,
        *,
        descending: bool = ...,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> OrderBy: ...
    def desc(
        self,
        *,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> OrderBy: ...
    def deconstruct(self) -> Any: ...  # fake

class ResolvedOuterRef(F): ...

class OuterRef(F):
    def __init__(self, name: str | OuterRef) -> None: ...
    contains_aggregate: bool
    def relabeled_clone(self: Self, relabels: Any) -> Self: ...

class Func(SQLiteNumericMixin, Expression):
    function: str
    name: str
    template: str
    arg_joiner: str
    arity: int | None
    source_expressions: list[Expression]
    extra: dict[Any, Any]
    def __init__(self, *expressions: Any, output_field: Field | None = ..., **extra: Any) -> None: ...
    def as_sql(
        self,
        compiler: SQLCompiler,
        connection: BaseDatabaseWrapper,
        function: str | None = ...,
        template: str | None = ...,
        arg_joiner: str | None = ...,
        **extra_context: Any,
    ) -> _AsSqlType: ...

class Value(Expression):
    value: Any
    def __init__(self, value: Any, output_field: Field | None = ...) -> None: ...

class RawSQL(Expression):
    params: list[Any]
    sql: str
    def __init__(self, sql: str, params: Sequence[Any], output_field: Field | None = ...) -> None: ...

class Star(Expression): ...

class Col(Expression):
    target: Field
    alias: str
    contains_column_references: Literal[True]
    possibly_multivalued: Literal[False]
    def __init__(self, alias: str, target: Field, output_field: Field | None = ...) -> None: ...

class Ref(Expression):
    def __init__(self, refs: str, source: Expression) -> None: ...

class ExpressionList(Func):
    def __init__(self, *expressions: BaseExpression | Combinable, **extra: Any) -> None: ...

class OrderByList(Func): ...

class ExpressionWrapper(Expression):
    def __init__(self, expression: Q | Combinable, output_field: Field) -> None: ...

class When(Expression):
    template: str
    condition: Any
    result: Any
    def __init__(self, condition: Any = ..., then: Any = ..., **lookups: Any) -> None: ...

class Case(Expression):
    template: str
    case_joiner: str
    cases: Any
    default: Any
    extra: Any
    def __init__(
        self, *cases: Any, default: Any | None = ..., output_field: Field | None = ..., **extra: Any
    ) -> None: ...

class Subquery(BaseExpression, Combinable):
    template: str
    query: Query
    extra: dict[Any, Any]
    def __init__(self, queryset: Query | QuerySet, output_field: Field | None = ..., **extra: Any) -> None: ...

class Exists(Subquery):
    negated: bool
    def __init__(self, queryset: Query | QuerySet, negated: bool = ..., **kwargs: Any) -> None: ...
    def __invert__(self) -> Exists: ...

class OrderBy(Expression):
    template: str
    nulls_first: bool
    nulls_last: bool
    descending: bool
    expression: Expression | F | Subquery
    def __init__(
        self,
        expression: Expression | F | Subquery,
        descending: bool = ...,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> None: ...

class Window(SQLiteNumericMixin, Expression):
    template: str
    contains_aggregate: bool
    contains_over_clause: bool
    partition_by: ExpressionList | None
    order_by: ExpressionList | None
    def __init__(
        self,
        expression: BaseExpression,
        partition_by: str | Iterable[BaseExpression | F] | F | BaseExpression | None = ...,
        order_by: Sequence[BaseExpression | F] | BaseExpression | F | None = ...,
        frame: WindowFrame | None = ...,
        output_field: Field | None = ...,
    ) -> None: ...

class WindowFrame(Expression):
    template: str
    frame_type: str
    def __init__(self, start: int | None = ..., end: int | None = ...) -> None: ...
    def window_frame_start_end(
        self, connection: BaseDatabaseWrapper, start: int | None, end: int | None
    ) -> tuple[int, int]: ...

class RowRange(WindowFrame): ...
class ValueRange(WindowFrame): ...



from typing import Any

from django.db.models import Func

class IntervalToSeconds(Func):
    function: str
    template: str
    def __init__(self, expression: Any, *, output_field: Any | None = ..., **extra: Any) -> None: ...

class SecondsToInterval(Func):
    function: str
    template: str
    def __init__(self, expression: Any, *, output_field: Any | None = ..., **extra: Any) -> None: ...



from collections.abc import Iterator, Sequence
from contextlib import AbstractContextManager, contextmanager
from types import TracebackType
from typing import Any

from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import AnonymousUser
from django.http import HttpRequest
from django.http.request import _ImmutableQueryDict
from rest_framework.authentication import BaseAuthentication
from rest_framework.authtoken.models import Token
from rest_framework.negotiation import BaseContentNegotiation
from rest_framework.parsers import BaseParser
from rest_framework.versioning import BaseVersioning
from rest_framework.views import APIView

def is_form_media_type(media_type: str) -> bool: ...

class override_method(AbstractContextManager[Request]):
    def __init__(self, view: APIView, request: Request, method: str): ...
    def __enter__(self) -> Request: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: TracebackType | None,
    ) -> bool | None: ...

class WrappedAttributeError(Exception): ...

@contextmanager
def wrap_attributeerrors() -> Iterator[None]: ...

class Empty: ...

def clone_request(request: Request, method: str) -> Request: ...

class ForcedAuthentication:
    force_user: AnonymousUser | AbstractBaseUser | None
    force_token: str | None
    def __init__(self, force_user: AnonymousUser | AbstractBaseUser | None, force_token: str | None) -> None: ...
    def authenticate(self, request: Request) -> tuple[AnonymousUser | AbstractBaseUser | None, Any | None]: ...

class Request(HttpRequest):
    parsers: Sequence[BaseParser] | None
    authenticators: Sequence[BaseAuthentication | ForcedAuthentication] | None
    negotiator: BaseContentNegotiation | None
    parser_context: dict[str, Any] | None
    version: str | None
    versioning_scheme: BaseVersioning | None
    _request: HttpRequest
    def __init__(
        self,
        request: HttpRequest,
        parsers: Sequence[BaseParser] | None = ...,
        authenticators: Sequence[BaseAuthentication] | None = ...,
        negotiator: BaseContentNegotiation | None = ...,
        parser_context: dict[str, Any] | None = ...,
    ) -> None: ...
    @property
    def content_type(self) -> str: ...  # type: ignore[override]
    @property
    def stream(self) -> Any: ...
    @property
    def query_params(self) -> _ImmutableQueryDict: ...
    @property
    def data(self) -> dict[str, Any]: ...
    @property  # type: ignore[override]
    def user(self) -> AbstractBaseUser | AnonymousUser: ...  # type: ignore[override]
    @user.setter
    def user(self, value: AbstractBaseUser | AnonymousUser) -> None: ...
    @property
    def auth(self) -> Token | Any: ...
    @auth.setter
    def auth(self, value: Token | Any) -> None: ...
    @property
    def successful_authenticator(self) -> BaseAuthentication | ForcedAuthentication | None: ...
    def __getattr__(self, attr: str) -> Any: ...
    @property
    def DATA(self) -> None: ...
    @property
    def POST(self) -> _ImmutableQueryDict: ...  # type: ignore[override]
    @property
    def FILES(self): ...
    @property
    def QUERY_PARAMS(self) -> None: ...
    def force_plaintext_errors(self, value: Any) -> None: ...



from typing import List, Union

from django.db.models import Prefetch

from nwon_django_toolbox.serializer.prefetch_relations import prefetch_relations


class EagerLoadingMixin:
    SELECT: List[str]
    PREFETCH: List[Union[str, Prefetch]]

    @classmethod
    def setup_eager_loading(cls, queryset):
        queryset = prefetch_relations(queryset)

        if hasattr(cls, "SELECT"):
            queryset = queryset.select_related(*cls.SELECT)

        if hasattr(cls, "PREFETCH"):
            queryset = queryset.prefetch_related(*cls.PREFETCH)

        return queryset



import sys
from tkinter import Event, Misc, Tk, Widget
from typing import ClassVar, Protocol

if sys.version_info >= (3, 9):
    __all__ = ["dnd_start", "DndHandler"]

class _DndSource(Protocol):
    def dnd_end(self, __target: Widget | None, __event: Event[Misc] | None) -> None: ...

class DndHandler:
    root: ClassVar[Tk | None]
    def __init__(self, source: _DndSource, event: Event[Misc]) -> None: ...
    def cancel(self, event: Event[Misc] | None = None) -> None: ...
    def finish(self, event: Event[Misc] | None, commit: int = 0) -> None: ...
    def on_motion(self, event: Event[Misc]) -> None: ...
    def on_release(self, event: Event[Misc]) -> None: ...
    def __del__(self) -> None: ...

def dnd_start(source: _DndSource, event: Event[Misc]) -> DndHandler | None: ...



from nwon_django_toolbox.typings.allowed_update import AllowedUpdate
from nwon_django_toolbox.typings.celery import CeleryFolder, CeleryReturn
from nwon_django_toolbox.typings.django_ready_enum import DjangoReadyEnum
from nwon_django_toolbox.typings.error_response import ErrorResponse
from nwon_django_toolbox.typings.permission_test_expectation import (
    LoginFunction,
    PermissionTestExpectation,
)
from nwon_django_toolbox.typings.permissions import (
    GroupPermissionMapping,
    PermissionConfiguration,
    PermissionConfigurationForGroup,
    PermissionPrefix,
)
from nwon_django_toolbox.typings.pydantic_base_django import PydanticBaseDjango
from nwon_django_toolbox.typings.request_body_format import RequestBodyFormat
from nwon_django_toolbox.typings.seed_set import SeedSet
from nwon_django_toolbox.typings.test_fixture import Fixture

__all__ = [
    "ErrorResponse",
    "SeedSet",
    "CeleryFolder",
    "CeleryReturn",
    "Fixture",
    "DjangoReadyEnum",
    "PydanticBaseDjango",
    "RequestBodyFormat",
    "PermissionTestExpectation",
    "LoginFunction",
    "GroupPermissionMapping",
    "PermissionConfiguration",
    "PermissionConfigurationForGroup",
    "PermissionPrefix",
    "AllowedUpdate",
]



import sys
from _typeshed import StrPath
from collections.abc import Mapping

LC_CTYPE: int
LC_COLLATE: int
LC_TIME: int
LC_MONETARY: int
LC_NUMERIC: int
LC_ALL: int
CHAR_MAX: int

def setlocale(__category: int, __locale: str | None = None) -> str: ...
def localeconv() -> Mapping[str, int | str | list[int]]: ...

if sys.version_info >= (3, 11):
    def getencoding() -> str: ...

def strcoll(__os1: str, __os2: str) -> int: ...
def strxfrm(__string: str) -> str: ...

# native gettext functions
# https://docs.python.org/3/library/locale.html#access-to-message-catalogs
# https://github.com/python/cpython/blob/f4c03484da59049eb62a9bf7777b963e2267d187/Modules/_localemodule.c#L626
if sys.platform != "win32":
    LC_MESSAGES: int

    ABDAY_1: int
    ABDAY_2: int
    ABDAY_3: int
    ABDAY_4: int
    ABDAY_5: int
    ABDAY_6: int
    ABDAY_7: int

    ABMON_1: int
    ABMON_2: int
    ABMON_3: int
    ABMON_4: int
    ABMON_5: int
    ABMON_6: int
    ABMON_7: int
    ABMON_8: int
    ABMON_9: int
    ABMON_10: int
    ABMON_11: int
    ABMON_12: int

    DAY_1: int
    DAY_2: int
    DAY_3: int
    DAY_4: int
    DAY_5: int
    DAY_6: int
    DAY_7: int

    ERA: int
    ERA_D_T_FMT: int
    ERA_D_FMT: int
    ERA_T_FMT: int

    MON_1: int
    MON_2: int
    MON_3: int
    MON_4: int
    MON_5: int
    MON_6: int
    MON_7: int
    MON_8: int
    MON_9: int
    MON_10: int
    MON_11: int
    MON_12: int

    CODESET: int
    D_T_FMT: int
    D_FMT: int
    T_FMT: int
    T_FMT_AMPM: int
    AM_STR: int
    PM_STR: int

    RADIXCHAR: int
    THOUSEP: int
    YESEXPR: int
    NOEXPR: int
    CRNCYSTR: int
    ALT_DIGITS: int

    def nl_langinfo(__key: int) -> str: ...

    # This is dependent on `libintl.h` which is a part of `gettext`
    # system dependency. These functions might be missing.
    # But, we always say that they are present.
    def gettext(__msg: str) -> str: ...
    def dgettext(__domain: str | None, __msg: str) -> str: ...
    def dcgettext(__domain: str | None, __msg: str, __category: int) -> str: ...
    def textdomain(__domain: str | None) -> str: ...
    def bindtextdomain(__domain: str, __dir: StrPath | None) -> str: ...
    def bind_textdomain_codeset(__domain: str, __codeset: str | None) -> str | None: ...



from typing import Any

from django.db.backends.base.validation import BaseDatabaseValidation
from django.db.backends.oracle.base import DatabaseWrapper

class DatabaseValidation(BaseDatabaseValidation):
    connection: DatabaseWrapper
    def check_field_type(self, field: Any, field_type: Any) -> Any: ...



from collections.abc import Iterable, Iterator, Sequence
from typing import Any, overload

from django.core.checks.messages import CheckMessage
from django.core.files.storage import FileSystemStorage, Storage
from typing_extensions import Literal

searched_locations: Any

class BaseFinder:
    def check(self, **kwargs: Any) -> list[CheckMessage]: ...
    @overload
    def find(self, path: str, all: Literal[False] = ...) -> str | None: ...
    @overload
    def find(self, path: str, all: Literal[True]) -> list[str]: ...
    def list(self, ignore_patterns: Iterable[str] | None) -> Iterable[Any]: ...

class FileSystemFinder(BaseFinder):
    locations: list[tuple[str, str]]
    storages: dict[str, Any]
    def __init__(self, app_names: Sequence[str] = ..., *args: Any, **kwargs: Any) -> None: ...
    def find_location(self, root: str, path: str, prefix: str | None = ...) -> str | None: ...
    @overload
    def find(self, path: str, all: Literal[False] = ...) -> str | None: ...
    @overload
    def find(self, path: str, all: Literal[True]) -> list[str]: ...
    def list(self, ignore_patterns: Iterable[str] | None) -> Iterable[Any]: ...

class AppDirectoriesFinder(BaseFinder):
    storage_class: type[FileSystemStorage]
    source_dir: str
    apps: list[str]
    storages: dict[str, FileSystemStorage]
    def __init__(self, app_names: Iterable[str] | None = ..., *args: Any, **kwargs: Any) -> None: ...
    def find_in_app(self, app: str, path: str) -> str | None: ...
    @overload
    def find(self, path: str, all: Literal[False] = ...) -> str | None: ...
    @overload
    def find(self, path: str, all: Literal[True]) -> list[str]: ...
    def list(self, ignore_patterns: Iterable[str] | None) -> Iterable[Any]: ...

class BaseStorageFinder(BaseFinder):
    storage: Storage
    def __init__(self, storage: Storage | None = ..., *args: Any, **kwargs: Any) -> None: ...
    @overload
    def find(self, path: str, all: Literal[False] = ...) -> str | None: ...
    @overload
    def find(self, path: str, all: Literal[True]) -> list[str]: ...
    def list(self, ignore_patterns: Iterable[str] | None) -> Iterable[Any]: ...

class DefaultStorageFinder(BaseStorageFinder):
    storage: Storage
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

@overload
def find(path: str, all: Literal[False] = ...) -> str | None: ...
@overload
def find(path: str, all: Literal[True]) -> list[str]: ...
def get_finders() -> Iterator[BaseFinder]: ...
@overload
def get_finder(
    import_path: Literal["django.contrib.staticfiles.finders.FileSystemFinder"],
) -> FileSystemFinder: ...
@overload
def get_finder(
    import_path: Literal["django.contrib.staticfiles.finders.AppDirectoriesFinder"],
) -> AppDirectoriesFinder: ...



from typing import Any

c_int_p: Any
register_all: Any
cleanup_all: Any
get_driver: Any
get_driver_by_name: Any
get_driver_count: Any
get_driver_name: Any
open_ds: Any
destroy_ds: Any
release_ds: Any
get_ds_name: Any
get_layer: Any
get_layer_by_name: Any
get_layer_count: Any
get_extent: Any
get_feature: Any
get_feature_count: Any
get_layer_defn: Any
get_layer_srs: Any
get_next_feature: Any
reset_reading: Any
test_capability: Any
get_spatial_filter: Any
set_spatial_filter: Any
set_spatial_filter_rect: Any
get_fd_geom_type: Any
get_fd_name: Any
get_feat_name: Any
get_field_count: Any
get_field_defn: Any
clone_feature: Any
destroy_feature: Any
feature_equal: Any
get_feat_geom_ref: Any
get_feat_field_count: Any
get_feat_field_defn: Any
get_fid: Any
get_field_as_datetime: Any
get_field_as_double: Any
get_field_as_integer: Any
get_field_as_integer64: Any
is_field_set: Any
get_field_as_string: Any
get_field_index: Any
get_field_name: Any
get_field_precision: Any
get_field_type: Any
get_field_type_name: Any
get_field_width: Any



import sys
from collections.abc import Iterable, Mapping
from typing import Any, Generic, TypeVar, overload
from typing_extensions import TypeAlias

if sys.version_info >= (3, 9):
    from types import GenericAlias

__all__ = ["CookieError", "BaseCookie", "SimpleCookie"]

_DataType: TypeAlias = str | Mapping[str, str | Morsel[Any]]
_T = TypeVar("_T")

@overload
def _quote(str: None) -> None: ...
@overload
def _quote(str: str) -> str: ...
@overload
def _unquote(str: None) -> None: ...
@overload
def _unquote(str: str) -> str: ...

class CookieError(Exception): ...

class Morsel(dict[str, Any], Generic[_T]):
    @property
    def value(self) -> str: ...
    @property
    def coded_value(self) -> _T: ...
    @property
    def key(self) -> str: ...
    def __init__(self) -> None: ...
    def set(self, key: str, val: str, coded_val: _T) -> None: ...
    def setdefault(self, key: str, val: str | None = None) -> str: ...
    # The dict update can also get a keywords argument so this is incompatible
    @overload  # type: ignore[override]
    def update(self, values: Mapping[str, str]) -> None: ...
    @overload
    def update(self, values: Iterable[tuple[str, str]]) -> None: ...
    def isReservedKey(self, K: str) -> bool: ...
    def output(self, attrs: list[str] | None = None, header: str = "Set-Cookie:") -> str: ...
    __str__ = output
    def js_output(self, attrs: list[str] | None = None) -> str: ...
    def OutputString(self, attrs: list[str] | None = None) -> str: ...
    def __eq__(self, morsel: object) -> bool: ...
    def __setitem__(self, K: str, V: Any) -> None: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class BaseCookie(dict[str, Morsel[_T]], Generic[_T]):
    def __init__(self, input: _DataType | None = None) -> None: ...
    def value_decode(self, val: str) -> tuple[_T, str]: ...
    def value_encode(self, val: _T) -> tuple[_T, str]: ...
    def output(self, attrs: list[str] | None = None, header: str = "Set-Cookie:", sep: str = "\r\n") -> str: ...
    __str__ = output
    def js_output(self, attrs: list[str] | None = None) -> str: ...
    def load(self, rawdata: _DataType) -> None: ...
    def __setitem__(self, key: str, value: str | Morsel[_T]) -> None: ...

class SimpleCookie(BaseCookie[str]): ...



__all__ = ["UnknownTimeZoneError", "InvalidTimeError", "AmbiguousTimeError", "NonExistentTimeError"]

class Error(Exception): ...
class UnknownTimeZoneError(KeyError, Error): ...
class InvalidTimeError(Error): ...
class AmbiguousTimeError(InvalidTimeError): ...
class NonExistentTimeError(InvalidTimeError): ...



from typing import Type, Union

from django.db.models import Model
from django.urls import reverse
from django.urls.exceptions import NoReverseMatch


def detail_url_for_model(model: Model, primary_key: Union[str, int, None] = None):
    used_primary_key: Union[str, int] = primary_key if primary_key else model.pk
    return detail_url_for_model_class(model.__class__, used_primary_key)


def detail_url_for_model_class(model: Type[Model], primary_key: Union[str, int]):
    view_name = _base_name_from_model_class(model) + "-detail"

    try:
        url = reverse(
            viewname=view_name,
            kwargs={"pk": primary_key},
        )
    except NoReverseMatch as exception:
        """
        Pretty project specific workaround to make this work with a custom user model
        named CustomUser
        """
        if view_name == "customuser-detail":
            return f"/users/{primary_key}/"

        # Fix for polymorphic child models that don't have their own view
        # We try to find the base model and use its detail view
        if hasattr(model, "__bases__"):
            class_to_use = model.__bases__[0]
            if class_to_use != model:
                return detail_url_for_model_class(class_to_use, primary_key)

        raise exception

    return url


def list_url_for_model_class(model_class: Type[Model]):
    try:
        return reverse(_base_name_from_model_class(model_class) + "-list")
    except NoReverseMatch as exception:
        # Fix for polymorphic child models that don't have their own view
        # We try to find the base model and use its list view
        if hasattr(model_class, "__bases__"):
            class_to_use = model_class.__bases__[0]
            if class_to_use != model_class:
                return list_url_for_model_class(class_to_use)

        raise exception


def list_url_for_model(model: Model):
    return list_url_for_model_class(model.__class__)


def _base_name_from_model_class(model_class: Type[Model]) -> str:
    return model_class.__name__.lower()


def base_name_from_model(model: Model) -> str:
    return _base_name_from_model_class(model.__class__)


__all__ = [
    "detail_url_for_model",
    "detail_url_for_model_class",
    "list_url_for_model_class",
    "list_url_for_model",
    "base_name_from_model",
]



from . import defaultfilters as defaultfilters

# Template parts
from .base import Node as Node
from .base import NodeList as NodeList
from .base import Origin as Origin
from .base import Template as Template
from .base import Variable as Variable
from .base import VariableDoesNotExist as VariableDoesNotExist
from .context import Context as Context
from .context import ContextPopException as ContextPopException
from .context import RequestContext as RequestContext
from .engine import Engine as Engine
from .exceptions import TemplateDoesNotExist as TemplateDoesNotExist
from .exceptions import TemplateSyntaxError as TemplateSyntaxError
from .library import Library as Library
from .utils import EngineHandler as EngineHandler

engines: EngineHandler



from collections.abc import Iterator
from pathlib import Path

from django.template.base import Origin
from django.template.engine import Engine

from .base import Loader as BaseLoader

class Loader(BaseLoader):
    dirs: list[str | Path] | None
    def __init__(self, engine: Engine, dirs: list[str | Path] | None = ...) -> None: ...
    def get_dirs(self) -> list[str | Path]: ...
    def get_contents(self, origin: Origin) -> str: ...
    def get_template_sources(self, template_name: str) -> Iterator[Origin]: ...



from typing import BinaryIO
from typing_extensions import TypeAlias

__all__ = ["Error", "encode", "decode"]

_File: TypeAlias = str | BinaryIO

class Error(Exception): ...

def encode(
    in_file: _File, out_file: _File, name: str | None = None, mode: int | None = None, *, backtick: bool = False
) -> None: ...
def decode(in_file: _File, out_file: _File | None = None, mode: int | None = None, quiet: bool = False) -> None: ...



from collections.abc import Iterable, Mapping
from typing import Any

from django.core.validators import MaxLengthValidator, MaxValueValidator, MinLengthValidator, MinValueValidator

class ArrayMaxLengthValidator(MaxLengthValidator): ...
class ArrayMinLengthValidator(MinLengthValidator): ...

class KeysValidator:
    messages: dict[str, str]
    strict: bool
    def __init__(self, keys: Iterable[str], strict: bool = ..., messages: Mapping[str, str] | None = ...) -> None: ...
    def __call__(self, value: Any) -> None: ...

class RangeMaxValueValidator(MaxValueValidator): ...
class RangeMinValueValidator(MinValueValidator): ...



from typing import Any, Dict, Optional

from rest_framework.response import Response
from rest_framework.views import exception_handler

from nwon_django_toolbox.exception_handler.get_error_response import get_error_response


def api_exception_handler(
    exc: Exception, context: Dict[str, Any]
) -> Optional[Response]:
    """Custom API exception handler."""

    # Call REST framework's default exception handler first,
    # to get the standard error response.
    response = exception_handler(exc, context)

    if response is not None:
        response.data = get_error_response(
            status_code=response.status_code,
            details=response.data,
        )

    return response



from typing import Any
from typing_extensions import TypeAlias
from weakref import ReferenceType

__all__ = ["local"]
_LocalDict: TypeAlias = dict[Any, Any]

class _localimpl:
    key: str
    dicts: dict[int, tuple[ReferenceType[Any], _LocalDict]]
    def get_dict(self) -> _LocalDict: ...
    def create_dict(self) -> _LocalDict: ...

class local:
    def __getattribute__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __delattr__(self, name: str) -> None: ...



import ssl
import sys
from _typeshed import StrPath
from collections.abc import AsyncIterator, Awaitable, Callable, Iterable, Sequence
from typing import Any
from typing_extensions import Self, SupportsIndex, TypeAlias

from . import events, protocols, transports
from .base_events import Server

if sys.platform == "win32":
    if sys.version_info >= (3, 8):
        __all__ = ("StreamReader", "StreamWriter", "StreamReaderProtocol", "open_connection", "start_server")
    else:
        __all__ = (
            "StreamReader",
            "StreamWriter",
            "StreamReaderProtocol",
            "open_connection",
            "start_server",
            "IncompleteReadError",
            "LimitOverrunError",
        )
else:
    if sys.version_info >= (3, 8):
        __all__ = (
            "StreamReader",
            "StreamWriter",
            "StreamReaderProtocol",
            "open_connection",
            "start_server",
            "open_unix_connection",
            "start_unix_server",
        )
    else:
        __all__ = (
            "StreamReader",
            "StreamWriter",
            "StreamReaderProtocol",
            "open_connection",
            "start_server",
            "IncompleteReadError",
            "LimitOverrunError",
            "open_unix_connection",
            "start_unix_server",
        )

_ClientConnectedCallback: TypeAlias = Callable[[StreamReader, StreamWriter], Awaitable[None] | None]

if sys.version_info < (3, 8):
    class IncompleteReadError(EOFError):
        expected: int | None
        partial: bytes
        def __init__(self, partial: bytes, expected: int | None) -> None: ...

    class LimitOverrunError(Exception):
        consumed: int
        def __init__(self, message: str, consumed: int) -> None: ...

if sys.version_info >= (3, 10):
    async def open_connection(
        host: str | None = None,
        port: int | str | None = None,
        *,
        limit: int = 65536,
        ssl_handshake_timeout: float | None = ...,
        **kwds: Any,
    ) -> tuple[StreamReader, StreamWriter]: ...
    async def start_server(
        client_connected_cb: _ClientConnectedCallback,
        host: str | Sequence[str] | None = None,
        port: int | str | None = None,
        *,
        limit: int = 65536,
        ssl_handshake_timeout: float | None = ...,
        **kwds: Any,
    ) -> Server: ...

else:
    async def open_connection(
        host: str | None = None,
        port: int | str | None = None,
        *,
        loop: events.AbstractEventLoop | None = None,
        limit: int = 65536,
        ssl_handshake_timeout: float | None = ...,
        **kwds: Any,
    ) -> tuple[StreamReader, StreamWriter]: ...
    async def start_server(
        client_connected_cb: _ClientConnectedCallback,
        host: str | None = None,
        port: int | str | None = None,
        *,
        loop: events.AbstractEventLoop | None = None,
        limit: int = 65536,
        ssl_handshake_timeout: float | None = ...,
        **kwds: Any,
    ) -> Server: ...

if sys.platform != "win32":
    if sys.version_info >= (3, 10):
        async def open_unix_connection(
            path: StrPath | None = None, *, limit: int = 65536, **kwds: Any
        ) -> tuple[StreamReader, StreamWriter]: ...
        async def start_unix_server(
            client_connected_cb: _ClientConnectedCallback, path: StrPath | None = None, *, limit: int = 65536, **kwds: Any
        ) -> Server: ...
    else:
        async def open_unix_connection(
            path: StrPath | None = None, *, loop: events.AbstractEventLoop | None = None, limit: int = 65536, **kwds: Any
        ) -> tuple[StreamReader, StreamWriter]: ...
        async def start_unix_server(
            client_connected_cb: _ClientConnectedCallback,
            path: StrPath | None = None,
            *,
            loop: events.AbstractEventLoop | None = None,
            limit: int = 65536,
            **kwds: Any,
        ) -> Server: ...

class FlowControlMixin(protocols.Protocol):
    def __init__(self, loop: events.AbstractEventLoop | None = None) -> None: ...

class StreamReaderProtocol(FlowControlMixin, protocols.Protocol):
    def __init__(
        self,
        stream_reader: StreamReader,
        client_connected_cb: _ClientConnectedCallback | None = None,
        loop: events.AbstractEventLoop | None = None,
    ) -> None: ...
    def __del__(self) -> None: ...

class StreamWriter:
    def __init__(
        self,
        transport: transports.WriteTransport,
        protocol: protocols.BaseProtocol,
        reader: StreamReader | None,
        loop: events.AbstractEventLoop,
    ) -> None: ...
    @property
    def transport(self) -> transports.WriteTransport: ...
    def write(self, data: bytes | bytearray | memoryview) -> None: ...
    def writelines(self, data: Iterable[bytes | bytearray | memoryview]) -> None: ...
    def write_eof(self) -> None: ...
    def can_write_eof(self) -> bool: ...
    def close(self) -> None: ...
    def is_closing(self) -> bool: ...
    async def wait_closed(self) -> None: ...
    def get_extra_info(self, name: str, default: Any = None) -> Any: ...
    async def drain(self) -> None: ...
    if sys.version_info >= (3, 12):
        async def start_tls(
            self,
            sslcontext: ssl.SSLContext,
            *,
            server_hostname: str | None = None,
            ssl_handshake_timeout: float | None = None,
            ssl_shutdown_timeout: float | None = None,
        ) -> None: ...
    elif sys.version_info >= (3, 11):
        async def start_tls(
            self, sslcontext: ssl.SSLContext, *, server_hostname: str | None = None, ssl_handshake_timeout: float | None = None
        ) -> None: ...
    if sys.version_info >= (3, 11):
        def __del__(self) -> None: ...

class StreamReader(AsyncIterator[bytes]):
    def __init__(self, limit: int = 65536, loop: events.AbstractEventLoop | None = None) -> None: ...
    def exception(self) -> Exception: ...
    def set_exception(self, exc: Exception) -> None: ...
    def set_transport(self, transport: transports.BaseTransport) -> None: ...
    def feed_eof(self) -> None: ...
    def at_eof(self) -> bool: ...
    def feed_data(self, data: Iterable[SupportsIndex]) -> None: ...
    async def readline(self) -> bytes: ...
    # Can be any buffer that supports len(); consider changing to a Protocol if PEP 688 is accepted
    async def readuntil(self, separator: bytes | bytearray | memoryview = b"\n") -> bytes: ...
    async def read(self, n: int = -1) -> bytes: ...
    async def readexactly(self, n: int) -> bytes: ...
    def __aiter__(self) -> Self: ...
    async def __anext__(self) -> bytes: ...



from collections.abc import Iterator
from typing import IO, Any, AnyStr, Generic

from django.utils._os import _PathCompatible

def validate_file_name(name: _PathCompatible, allow_relative_path: bool = ...) -> _PathCompatible: ...

class FileProxyMixin(Generic[AnyStr]):
    file: IO[AnyStr] | None
    encoding: Any
    fileno: Any
    flush: Any
    isatty: Any
    newlines: Any
    read: Any
    readinto: Any
    readline: Any
    readlines: Any
    seek: Any
    tell: Any
    truncate: Any
    write: Any
    writelines: Any
    @property
    def closed(self) -> bool: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def seekable(self) -> bool: ...
    def __iter__(self) -> Iterator[AnyStr]: ...



from collections.abc import Iterable, Mapping, Sequence
from typing import Any
from typing_extensions import TypeAlias

from . import fields

RequestField = fields.RequestField

writer: Any

_TYPE_FIELDS_SEQUENCE: TypeAlias = Sequence[tuple[str, fields._FieldValueTuple] | RequestField]
_TYPE_FIELDS: TypeAlias = _TYPE_FIELDS_SEQUENCE | Mapping[str, fields._FieldValueTuple]

def choose_boundary() -> str: ...
def iter_field_objects(fields: _TYPE_FIELDS) -> Iterable[RequestField]: ...
def iter_fields(fields): ...
def encode_multipart_formdata(fields: _TYPE_FIELDS, boundary: str | None = None) -> tuple[bytes, str]: ...



from logging import Logger
from typing import Any

from celery.app.base import Celery
from celery.utils.log import LoggingProxy
from celery.utils.term import colored as colored_

class Logging:
    app: Celery
    loglevel: int
    format: str
    task_format: str
    colorize: bool
    def __init__(self, app: Celery) -> None: ...
    def setup(
        self,
        loglevel: int | None = ...,
        logfile: str | None = ...,
        redirect_stdouts: bool = ...,
        redirect_level: str = ...,
        colorize: bool | None = ...,
        hostname: str | None = ...,
    ) -> list[tuple[Any, Any]]: ...
    def redirect_stdouts(self, loglevel: int | None = ..., name: str = ...) -> None: ...
    def setup_logging_subsystem(
        self,
        loglevel: str | None = ...,
        logfile: Any | None = ...,
        format: str | None = ...,
        colorize: bool | None = ...,
        hostname: str | None = ...,
        **kwargs: Any
    ) -> Any: ...
    def setup_task_loggers(
        self,
        loglevel: int | str | None = ...,
        logfile: Any | None = ...,
        format: str | None = ...,
        colorize: bool | None = ...,
        propagate: bool = ...,
        **kwargs: Any
    ) -> Logger: ...
    def redirect_stdouts_to_logger(
        self,
        logger: Logger,
        loglevel: int | str | None = ...,
        stdout: bool = ...,
        stderr: bool = ...,
    ) -> LoggingProxy: ...
    def supports_color(
        self, colorize: bool | None = ..., logfile: Any | None = ...
    ) -> bool: ...
    def colored(
        self, logfile: Any | None = ..., enabled: bool | None = ...
    ) -> colored_: ...
    def setup_handlers(
        self,
        logger: Logger,
        logfile: Any,
        format: str,
        colorize: bool,
        formatter: Any = ...,
        **kwargs: Any
    ) -> Logger: ...
    def get_default_logger(self, name: str = ..., **kwargs: Any) -> Logger: ...



import datetime
from collections.abc import Callable, Iterable, Sequence
from typing import Any, overload
from uuid import UUID

from django.contrib.admin.options import BaseModelAdmin
from django.contrib.admin.sites import AdminSite
from django.contrib.auth.forms import AdminPasswordChangeForm
from django.db.models.base import Model
from django.db.models.deletion import Collector
from django.db.models.fields import Field, reverse_related
from django.db.models.options import Options
from django.db.models.query import QuerySet
from django.forms.forms import BaseForm
from django.forms.formsets import BaseFormSet
from django.http.request import HttpRequest
from django.utils.datastructures import _IndexableCollection
from typing_extensions import Literal, TypedDict

class FieldIsAForeignKeyColumnName(Exception): ...

def lookup_needs_distinct(opts: Options, lookup_path: str) -> bool: ...
def prepare_lookup_value(key: str, value: datetime.datetime | str) -> bool | datetime.datetime | str: ...
def quote(s: int | str | UUID) -> str: ...
def unquote(s: str) -> str: ...
def flatten(fields: Any) -> list[Callable | str]: ...
def flatten_fieldsets(fieldsets: Any) -> list[Callable | str]: ...
def get_deleted_objects(
    objs: Sequence[Model | None] | QuerySet[Model], request: HttpRequest, admin_site: AdminSite
) -> tuple[list[Model], dict[str, int], set[str], list[str]]: ...

class NestedObjects(Collector):
    data: dict[type[Model], set[Model] | list[Model]]
    dependencies: dict[Any, Any]
    fast_deletes: list[Any]
    field_updates: dict[Any, Any]
    using: str
    edges: Any
    protected: Any
    model_objs: Any
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def add_edge(self, source: Model | None, target: Model) -> None: ...
    def related_objects(
        self, related_model: type[Model], related_fields: Iterable[Field], objs: _IndexableCollection[Model]
    ) -> QuerySet[Model]: ...
    def nested(self, format_callback: Callable = ...) -> list[Any]: ...
    def can_fast_delete(self, *args: Any, **kwargs: Any) -> bool: ...

class _ModelFormatDict(TypedDict):
    verbose_name: str
    verbose_name_plural: str

def model_format_dict(obj: Model | type[Model] | QuerySet | Options[Model]) -> _ModelFormatDict: ...
def model_ngettext(obj: Options | QuerySet, n: int | None = ...) -> str: ...
def lookup_field(
    name: Callable | str, obj: Model, model_admin: BaseModelAdmin | None = ...
) -> tuple[Field | None, str | None, Any]: ...
@overload
def label_for_field(  # type: ignore
    name: Callable | str,
    model: type[Model],
    model_admin: BaseModelAdmin | None = ...,
    return_attr: Literal[True] = ...,
    form: BaseForm | None = ...,
) -> tuple[str, Callable | str | None]: ...
@overload
def label_for_field(
    name: Callable | str,
    model: type[Model],
    model_admin: BaseModelAdmin | None = ...,
    return_attr: Literal[False] = ...,
    form: BaseForm | None = ...,
) -> str: ...
def help_text_for_field(name: str, model: type[Model]) -> str: ...
def display_for_field(value: Any, field: Field, empty_value_display: str) -> str: ...
def display_for_value(value: Any, empty_value_display: str, boolean: bool = ...) -> str: ...

class NotRelationField(Exception): ...

def get_model_from_relation(field: Field | reverse_related.ForeignObjectRel) -> type[Model]: ...
def reverse_field_path(model: type[Model], path: str) -> tuple[type[Model], str]: ...
def get_fields_from_path(model: type[Model], path: str) -> list[Field]: ...
def construct_change_message(
    form: AdminPasswordChangeForm, formsets: Iterable[BaseFormSet], add: bool
) -> list[dict[str, dict[str, list[str]]]]: ...



from collections.abc import Callable, Sequence
from typing import Any

from django.db.models.base import Model
from django.db.models.fields import AutoField, Field, _AllLimitChoicesTo, _ChoicesList, _LimitChoicesTo
from django.db.models.fields.related import ForeignKey, ForeignObject, ManyToManyField, OneToOneField
from django.db.models.lookups import Lookup, StartsWith
from django.db.models.query_utils import FilteredRelation, PathInfo
from django.db.models.sql.where import WhereNode
from typing_extensions import Literal

from .mixins import FieldCacheMixin

# Common note: `model` and `through` can be of type `str` when passed to `__init__`.
# When parent's `contribute_to_class` is called (during startup),
# strings are resolved to real model classes.
# Thus `str` is acceptable in __init__, but instance attribute `model` is always
# `Type[Model]`

class ForeignObjectRel(FieldCacheMixin):
    auto_created: bool
    concrete: Literal[False]
    editable: bool
    is_relation: bool
    null: bool
    field: ForeignObject
    model: type[Model]
    related_name: str | None
    related_query_name: str | None
    limit_choices_to: _AllLimitChoicesTo | None
    parent_link: bool
    on_delete: Callable
    symmetrical: bool
    multiple: bool
    field_name: str | None
    def __init__(
        self,
        field: ForeignObject,
        to: type[Model] | str,
        related_name: str | None = ...,
        related_query_name: str | None = ...,
        limit_choices_to: _AllLimitChoicesTo | None = ...,
        parent_link: bool = ...,
        on_delete: Callable = ...,
    ) -> None: ...
    @property
    def hidden(self) -> bool: ...
    @property
    def name(self) -> str: ...
    @property
    def remote_field(self) -> ForeignObject: ...
    @property
    def target_field(self) -> AutoField: ...
    @property
    def related_model(self) -> type[Model]: ...
    @property
    def many_to_many(self) -> bool: ...
    @property
    def many_to_one(self) -> bool: ...
    @property
    def one_to_many(self) -> bool: ...
    @property
    def one_to_one(self) -> bool: ...
    def get_lookup(self, lookup_name: str) -> type[Lookup] | None: ...
    def get_internal_type(self) -> str: ...
    @property
    def db_type(self) -> Any: ...
    # Yes, seems that `get_choices` will fail if `limit_choices_to=None`
    # and `self.limit_choices_to` is callable.
    def get_choices(
        self,
        include_blank: bool = ...,
        blank_choice: _ChoicesList = ...,
        limit_choices_to: _LimitChoicesTo | None = ...,
        ordering: Sequence[str] = ...,
    ) -> _ChoicesList: ...
    def is_hidden(self) -> bool: ...
    def get_joining_columns(self) -> tuple: ...
    def get_extra_restriction(
        self, where_class: type[WhereNode], alias: str, related_alias: str
    ) -> StartsWith | WhereNode | None: ...
    def set_field_name(self) -> None: ...
    def get_accessor_name(self, model: type[Model] | None = ...) -> str | None: ...
    def get_path_info(self, filtered_relation: FilteredRelation | None = ...) -> list[PathInfo]: ...

class ManyToOneRel(ForeignObjectRel):
    field: ForeignKey
    def __init__(
        self,
        field: ForeignKey,
        to: type[Model] | str,
        field_name: str,
        related_name: str | None = ...,
        related_query_name: str | None = ...,
        limit_choices_to: _AllLimitChoicesTo | None = ...,
        parent_link: bool = ...,
        on_delete: Callable = ...,
    ) -> None: ...
    def get_related_field(self) -> Field: ...

class OneToOneRel(ManyToOneRel):
    field: OneToOneField
    def __init__(
        self,
        field: OneToOneField,
        to: type[Model] | str,
        field_name: str | None,
        related_name: str | None = ...,
        related_query_name: str | None = ...,
        limit_choices_to: _AllLimitChoicesTo | None = ...,
        parent_link: bool = ...,
        on_delete: Callable = ...,
    ) -> None: ...

class ManyToManyRel(ForeignObjectRel):
    field: ManyToManyField  # type: ignore
    through: type[Model] | None
    through_fields: tuple[str, str] | None
    db_constraint: bool
    def __init__(
        self,
        field: ManyToManyField,
        to: type[Model] | str,
        related_name: str | None = ...,
        related_query_name: str | None = ...,
        limit_choices_to: _AllLimitChoicesTo | None = ...,
        symmetrical: bool = ...,
        through: type[Model] | str | None = ...,
        through_fields: tuple[str, str] | None = ...,
        db_constraint: bool = ...,
    ) -> None: ...
    def get_related_field(self) -> Field: ...



import datetime
from decimal import Decimal
from typing import Any, TypeVar, overload

from django.utils.functional import Promise
from typing_extensions import Literal, TypeGuard

class DjangoUnicodeDecodeError(UnicodeDecodeError):
    obj: bytes
    def __init__(self, obj: bytes, *args: Any) -> None: ...

_P = TypeVar("_P", bound=Promise)
_S = TypeVar("_S", bound=str)
_B = TypeVar("_B", bound=bytes)
_PT = TypeVar("_PT", None, int, float, Decimal, datetime.datetime, datetime.date, datetime.time)

@overload
def smart_str(s: _P, encoding: str = ..., strings_only: bool = ..., errors: str = ...) -> _P: ...
@overload
def smart_str(s: _S, encoding: str = ..., *, errors: str = ...) -> _S: ...
@overload
def smart_str(s: Any, encoding: str = ..., *, errors: str = ...) -> str: ...
@overload
def smart_str(s: _PT, encoding: str = ..., strings_only: Literal[True] = ..., errors: str = ...) -> _PT: ...
@overload
def smart_str(s: _S, encoding: str = ..., strings_only: bool = ..., errors: str = ...) -> _S: ...
@overload
def smart_str(s: Any, encoding: str = ..., strings_only: bool = ..., errors: str = ...) -> str: ...

smart_text = smart_str  # Deprecated

def is_protected_type(obj: Any) -> TypeGuard[_PT]: ...
@overload
def force_str(s: _S, encoding: str = ..., *, errors: str = ...) -> _S: ...
@overload
def force_str(s: Any, encoding: str = ..., *, errors: str = ...) -> str: ...
@overload
def force_str(s: _PT, encoding: str = ..., strings_only: Literal[True] = ..., errors: str = ...) -> _PT: ...
@overload
def force_str(s: _S, encoding: str = ..., strings_only: bool = ..., errors: str = ...) -> _S: ...
@overload
def force_str(s: Any, encoding: str = ..., strings_only: bool = ..., errors: str = ...) -> str: ...

force_text = force_str  # Deprecated

@overload
def smart_bytes(s: _P, encoding: str = ..., strings_only: bool = ..., errors: str = ...) -> _P: ...
@overload
def smart_bytes(s: _B, encoding: Literal["utf-8"] = ..., *, errors: str = ...) -> _B: ...
@overload
def smart_bytes(s: Any, encoding: str = ..., *, errors: str = ...) -> bytes: ...
@overload
def smart_bytes(s: _PT, encoding: str = ..., strings_only: Literal[True] = ..., errors: str = ...) -> _PT: ...
@overload
def smart_bytes(s: Any, encoding: str = ..., strings_only: bool = ..., errors: str = ...) -> bytes: ...
@overload
def force_bytes(s: _B, encoding: Literal["utf-8"] = ..., *, errors: str = ...) -> _B: ...
@overload
def force_bytes(s: Any, encoding: str = ..., *, errors: str = ...) -> bytes: ...
@overload
def force_bytes(s: _PT, encoding: str = ..., strings_only: Literal[True] = ..., errors: str = ...) -> _PT: ...
@overload
def force_bytes(s: Any, encoding: str = ..., strings_only: bool = ..., errors: str = ...) -> bytes: ...
@overload
def iri_to_uri(iri: None) -> None: ...
@overload
def iri_to_uri(iri: str | Promise) -> str: ...
@overload
def uri_to_iri(uri: None) -> None: ...  # type: ignore
@overload
def uri_to_iri(uri: Any) -> str: ...
def escape_uri_path(path: str) -> str: ...
def punycode(domain: str) -> str: ...
def repercent_broken_unicode(path: bytes) -> bytes: ...
@overload
def filepath_to_uri(path: None) -> None: ...
@overload
def filepath_to_uri(path: str) -> str: ...
def get_system_encoding() -> str: ...

DEFAULT_LOCALE_ENCODING: str



import builtins
import types
from _typeshed import ReadableBuffer, SupportsRead, SupportsWrite
from typing import Any
from typing_extensions import TypeAlias

version: int

_Marshallable: TypeAlias = (
    # handled in w_object() in marshal.c
    None
    | type[StopIteration]
    | builtins.ellipsis
    | bool
    # handled in w_complex_object() in marshal.c
    | int
    | float
    | complex
    | bytes
    | str
    | tuple[_Marshallable, ...]
    | list[Any]
    | dict[Any, Any]
    | set[Any]
    | frozenset[_Marshallable]
    | types.CodeType
    | ReadableBuffer
)

def dump(__value: _Marshallable, __file: SupportsWrite[bytes], __version: int = 4) -> None: ...
def load(__file: SupportsRead[bytes]) -> Any: ...
def dumps(__value: _Marshallable, __version: int = 4) -> bytes: ...
def loads(__bytes: ReadableBuffer) -> Any: ...



import socket
import sys
from _typeshed import Incomplete, ReadableBuffer, WriteableBuffer
from collections.abc import Callable
from typing import IO, Any, ClassVar, Literal, NoReturn

from . import events, futures, proactor_events, selector_events, streams, windows_utils

if sys.platform == "win32":
    __all__ = (
        "SelectorEventLoop",
        "ProactorEventLoop",
        "IocpProactor",
        "DefaultEventLoopPolicy",
        "WindowsSelectorEventLoopPolicy",
        "WindowsProactorEventLoopPolicy",
    )

    NULL: Literal[0]
    INFINITE: Literal[0xFFFFFFFF]
    ERROR_CONNECTION_REFUSED: Literal[1225]
    ERROR_CONNECTION_ABORTED: Literal[1236]
    CONNECT_PIPE_INIT_DELAY: float
    CONNECT_PIPE_MAX_DELAY: float

    class PipeServer:
        def __init__(self, address: str) -> None: ...
        def __del__(self) -> None: ...
        def closed(self) -> bool: ...
        def close(self) -> None: ...

    class _WindowsSelectorEventLoop(selector_events.BaseSelectorEventLoop): ...

    class ProactorEventLoop(proactor_events.BaseProactorEventLoop):
        def __init__(self, proactor: IocpProactor | None = None) -> None: ...
        async def create_pipe_connection(
            self, protocol_factory: Callable[[], streams.StreamReaderProtocol], address: str
        ) -> tuple[proactor_events._ProactorDuplexPipeTransport, streams.StreamReaderProtocol]: ...
        async def start_serving_pipe(
            self, protocol_factory: Callable[[], streams.StreamReaderProtocol], address: str
        ) -> list[PipeServer]: ...

    class IocpProactor:
        def __init__(self, concurrency: int = 0xFFFFFFFF) -> None: ...
        def __del__(self) -> None: ...
        def set_loop(self, loop: events.AbstractEventLoop) -> None: ...
        def select(self, timeout: int | None = None) -> list[futures.Future[Any]]: ...
        def recv(self, conn: socket.socket, nbytes: int, flags: int = 0) -> futures.Future[bytes]: ...
        def recv_into(self, conn: socket.socket, buf: WriteableBuffer, flags: int = 0) -> futures.Future[Any]: ...
        def recvfrom(
            self, conn: socket.socket, nbytes: int, flags: int = 0
        ) -> futures.Future[tuple[bytes, socket._RetAddress]]: ...
        def sendto(
            self, conn: socket.socket, buf: ReadableBuffer, flags: int = 0, addr: socket._Address | None = None
        ) -> futures.Future[int]: ...
        def send(self, conn: socket.socket, buf: WriteableBuffer, flags: int = 0) -> futures.Future[Any]: ...
        def accept(self, listener: socket.socket) -> futures.Future[Any]: ...
        def connect(
            self,
            conn: socket.socket,
            address: tuple[Incomplete, Incomplete] | tuple[Incomplete, Incomplete, Incomplete, Incomplete],
        ) -> futures.Future[Any]: ...
        def sendfile(self, sock: socket.socket, file: IO[bytes], offset: int, count: int) -> futures.Future[Any]: ...
        def accept_pipe(self, pipe: socket.socket) -> futures.Future[Any]: ...
        async def connect_pipe(self, address: str) -> windows_utils.PipeHandle: ...
        def wait_for_handle(self, handle: windows_utils.PipeHandle, timeout: int | None = None) -> bool: ...
        def close(self) -> None: ...
        if sys.version_info >= (3, 11):
            def recvfrom_into(
                self, conn: socket.socket, buf: WriteableBuffer, flags: int = 0
            ) -> futures.Future[tuple[int, socket._RetAddress]]: ...

    SelectorEventLoop = _WindowsSelectorEventLoop

    class WindowsSelectorEventLoopPolicy(events.BaseDefaultEventLoopPolicy):
        _loop_factory: ClassVar[type[SelectorEventLoop]]
        def get_child_watcher(self) -> NoReturn: ...
        def set_child_watcher(self, watcher: Any) -> NoReturn: ...

    class WindowsProactorEventLoopPolicy(events.BaseDefaultEventLoopPolicy):
        _loop_factory: ClassVar[type[ProactorEventLoop]]
        def get_child_watcher(self) -> NoReturn: ...
        def set_child_watcher(self, watcher: Any) -> NoReturn: ...

    DefaultEventLoopPolicy = WindowsSelectorEventLoopPolicy



from collections.abc import Callable, Iterable, Iterator
from contextlib import contextmanager
from types import TracebackType
from typing import Any, TypeVar

from _typeshed import Self
from django.http.request import HttpRequest
from django.template.base import Node, Origin, Template
from django.template.loader_tags import IncludeNode
from typing_extensions import TypeAlias

_ContextKeys: TypeAlias = int | str | Node

_ContextValues: TypeAlias = dict[str, Any] | Context

class ContextPopException(Exception): ...

class ContextDict(dict):
    context: BaseContext
    def __init__(self, context: BaseContext, *args: Any, **kwargs: Any) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...

class BaseContext(Iterable[Any]):
    def __init__(self, dict_: Any = ...) -> None: ...
    def __copy__(self: Self) -> Self: ...
    def __iter__(self) -> Iterator[Any]: ...
    def push(self, *args: Any, **kwargs: Any) -> ContextDict: ...
    def pop(self) -> ContextDict: ...
    def __setitem__(self, key: _ContextKeys, value: Any) -> None: ...
    def set_upward(self, key: _ContextKeys, value: int | str) -> None: ...
    def __getitem__(self, key: _ContextKeys) -> Any: ...
    def __delitem__(self, key: _ContextKeys) -> None: ...
    def __contains__(self, key: _ContextKeys) -> bool: ...
    def get(self, key: _ContextKeys, otherwise: Any | None = ...) -> Any | None: ...
    def setdefault(self, key: _ContextKeys, default: list[Origin] | int | None = ...) -> list[Origin] | int | None: ...
    def new(self, values: _ContextValues | None = ...) -> Context: ...
    def flatten(self) -> dict[_ContextKeys, dict[_ContextKeys, type[Any] | str] | int | str | None]: ...

class Context(BaseContext):
    dicts: Any
    autoescape: bool
    use_l10n: bool | None
    use_tz: bool | None
    template_name: str | None
    render_context: RenderContext
    template: Template | None
    def __init__(
        self, dict_: Any = ..., autoescape: bool = ..., use_l10n: bool | None = ..., use_tz: bool | None = ...
    ) -> None: ...
    @contextmanager
    def bind_template(self, template: Template) -> Iterator[None]: ...
    def update(self, other_dict: dict[str, Any] | Context) -> ContextDict: ...

class RenderContext(BaseContext):
    dicts: list[dict[IncludeNode | str, str]]
    template: Template | None
    @contextmanager
    def push_state(self, template: Template, isolated_context: bool = ...) -> Iterator[None]: ...

class RequestContext(Context):
    autoescape: bool
    dicts: list[dict[str, str]]
    render_context: RenderContext
    template_name: str | None
    use_l10n: bool | None
    use_tz: bool | None
    request: HttpRequest
    def __init__(
        self,
        request: HttpRequest,
        dict_: dict[str, Any] | None = ...,
        processors: list[Callable] | None = ...,
        use_l10n: bool | None = ...,
        use_tz: bool | None = ...,
        autoescape: bool = ...,
    ) -> None: ...
    template: Template | None
    @contextmanager
    def bind_template(self, template: Template) -> Iterator[None]: ...
    def new(self, values: _ContextValues | None = ...) -> RequestContext: ...

def make_context(context: Any, request: HttpRequest | None = ..., **kwargs: Any) -> Context: ...



from typing import Any, Final, TypeVar

_T = TypeVar("_T")

__about__: Final[str]

def heapify(heap: list[Any], /) -> None: ...
def heappop(heap: list[_T], /) -> _T: ...
def heappush(heap: list[_T], item: _T, /) -> None: ...
def heappushpop(heap: list[_T], item: _T, /) -> _T: ...
def heapreplace(heap: list[_T], item: _T, /) -> _T: ...



from collections.abc import Awaitable, Callable, Iterator, Mapping, Sequence
from typing import IO, Any, TypeVar

from django.core.handlers import base
from django.http.request import HttpRequest, _ImmutableQueryDict
from django.http.response import HttpResponseBase
from django.urls.resolvers import ResolverMatch, URLResolver
from django.utils.datastructures import MultiValueDict
from typing_extensions import TypeAlias

_ReceiveCallback: TypeAlias = Callable[[], Awaitable[Mapping[str, Any]]]

_SendCallback: TypeAlias = Callable[[Mapping[str, Any]], Awaitable[None]]

class ASGIRequest(HttpRequest):
    body_receive_timeout: int
    scope: Mapping[str, Any]
    resolver_match: ResolverMatch | None
    script_name: str | None
    path_info: str
    path: str
    method: str
    META: dict[str, Any]
    def __init__(self, scope: Mapping[str, Any], body_file: IO[bytes]) -> None: ...
    @property
    def GET(self) -> _ImmutableQueryDict: ...  # type: ignore
    POST: _ImmutableQueryDict
    FILES: MultiValueDict
    @property
    def COOKIES(self) -> dict[str, str]: ...  # type: ignore

_T = TypeVar("_T")

class ASGIHandler(base.BaseHandler):
    request_class: type[ASGIRequest]
    chunk_size: int
    def __init__(self) -> None: ...
    async def __call__(
        self,
        scope: dict[str, Any],
        receive: _ReceiveCallback,
        send: _SendCallback,
    ) -> None: ...
    async def read_body(self, receive: _ReceiveCallback) -> IO[bytes]: ...
    def create_request(
        self, scope: Mapping[str, Any], body_file: IO[bytes]
    ) -> tuple[ASGIRequest, None] | tuple[None, HttpResponseBase]: ...
    def handle_uncaught_exception(
        self, request: HttpRequest, resolver: URLResolver, exc_info: Any
    ) -> HttpResponseBase: ...
    async def send_response(self, response: HttpResponseBase, send: _SendCallback) -> None: ...
    @classmethod
    def chunk_bytes(cls, data: Sequence[_T]) -> Iterator[tuple[Sequence[_T], bool]]: ...
    def get_script_prefix(self, scope: Mapping[str, Any]) -> str: ...



from django.contrib.gis.db.backends.base.features import BaseSpatialFeatures
from django.db.backends.oracle.features import DatabaseFeatures as OracleDatabaseFeatures

class DatabaseFeatures(BaseSpatialFeatures, OracleDatabaseFeatures):
    supports_add_srs_entry: bool
    supports_geometry_field_introspection: bool
    supports_geometry_field_unique_index: bool
    supports_perimeter_geodetic: bool
    supports_dwithin_distance_expr: bool



from typing import Any, List, Optional, Type

from django.db.models.base import Model
from rest_framework.serializers import Serializer

from nwon_django_toolbox.tests.api_client.api_test import ApiTest
from nwon_django_toolbox.tests.helper.check_object_against_parameter import (
    check_object_against_parameter,
)
from nwon_django_toolbox.tests.helper.dictionary_is_serialized_instance import (
    dictionary_is_serialized_instance,
)
from nwon_django_toolbox.typings import RequestBodyFormat
from nwon_django_toolbox.url_helper import detail_url_for_model, list_url_for_model


def check_patch_basics(
    model: Model,
    authentication_token: Optional[str] = None,
    return_serializer: Optional[Type[Serializer]] = None,
    needs_authentication: bool = True,
):
    """
    Test basics on a patch endpoint.

    1. Unauthorized response without authentication and needs_authentication == True
    2. Not found on missing id
    3. Successful patch with empty body
    (4.) Optionally tests the response against a serializer
    """

    api_test = ApiTest()

    if needs_authentication:
        list_url = list_url_for_model(model)
        non_existing_detail_url = detail_url_for_model(model, "some-non-existing-id")
        detail_url = detail_url_for_model(model)

        api_test.patch_unauthorized(list_url, {})
        api_test.patch_unauthorized(non_existing_detail_url, {})
        api_test.patch_unauthorized(detail_url, {})

        if authentication_token:
            api_test.set_bearer_token(authentication_token)
            __basic_patch_test(api_test, model, return_serializer)
    else:
        __basic_patch_test(api_test, model, return_serializer)


def __basic_patch_test(
    api_test: ApiTest,
    model: Model,
    return_serializer: Optional[Type[Serializer]] = None,
):
    list_url = list_url_for_model(model)
    non_existing_detail_url = detail_url_for_model(model, "some-non-existing-id")
    detail_url = detail_url_for_model(model)

    api_test.patch_method_not_allowed(list_url, {})
    api_test.patch_not_found(non_existing_detail_url, {})
    response = api_test.patch_successful(detail_url, {})

    if return_serializer:
        model.refresh_from_db()
        assert dictionary_is_serialized_instance(
            instance=model,
            serializer=return_serializer,
            response=response,
        )


def check_patch_method_not_allowed(
    model: Model,
    authentication_token: Optional[str] = None,
):
    """
    Test that patch is not allowed
    """

    list_url = list_url_for_model(model)
    detail_url = detail_url_for_model(model)

    api_test = ApiTest(token=authentication_token)
    api_test.patch_method_not_allowed(list_url, {})
    api_test.patch_method_not_allowed(detail_url, {})


def check_patch_parameters_successful(
    model: Model,
    successful_parameters: List[dict],
    authentication_token: Optional[str] = None,
    return_serializer: Optional[Type[Serializer]] = None,
    body_format: RequestBodyFormat = RequestBodyFormat.Json,
):
    """
    Test that a list of parameter patches successfully.

    1. Successful patch
    2. Checks the request parameters against the response.
    (3.) Optionally tests the response against a serializer
    """

    url = detail_url_for_model(model)
    api_test = ApiTest(token=authentication_token)

    # test a successful patch
    for successful_patch_parameter in successful_parameters:
        response = api_test.patch_successful(
            url, successful_patch_parameter, body_format
        )
        check_object_against_parameter(response, successful_patch_parameter)

        if return_serializer:
            model.refresh_from_db()
            assert dictionary_is_serialized_instance(
                instance=model,
                serializer=return_serializer,
                response=response,
            )


def check_patch_parameters_failing(
    model: Model,
    failing_parameters: List[dict],
    authentication_token: Optional[str] = None,
    body_format: RequestBodyFormat = RequestBodyFormat.Json,
):
    """
    Test that a list of parameter fails
    """

    url = detail_url_for_model(model)
    api_test = ApiTest(token=authentication_token)

    # test a failing patch
    for failing_patch_parameter in failing_parameters:
        api_test.patch_bad_request(url, failing_patch_parameter, body_format)


def check_patch_read_only_field(
    model: Model,
    successful_patch_parameter: dict,
    key: str,
    value: Any,
    authentication_token: Optional[str] = None,
    body_format: RequestBodyFormat = RequestBodyFormat.Json,
):
    """
    Check that a patch request with a certain parameter does not have any effect
    """

    successful_patch_parameter[key] = value
    api_test = ApiTest(token=authentication_token)

    url = detail_url_for_model(model)
    updated_object = api_test.patch_successful(
        url, successful_patch_parameter, body_format
    )

    assert updated_object[key] != value


__all__ = [
    "check_patch_basics",
    "check_patch_method_not_allowed",
    "check_patch_parameters_failing",
    "check_patch_parameters_successful",
    "check_patch_read_only_field",
]



from collections.abc import Mapping, Sequence
from pathlib import Path, PosixPath
from typing import Any, Protocol

from django.db.models.base import Model
from typing_extensions import TypeAlias

_UserModel: TypeAlias = Model

class PasswordValidator(Protocol):
    def validate(self, __password: str, __user: _UserModel | None = ...) -> None: ...
    def get_help_text(self) -> str: ...

def get_default_password_validators() -> list[PasswordValidator]: ...
def get_password_validators(validator_config: Sequence[Mapping[str, Any]]) -> list[PasswordValidator]: ...
def validate_password(
    password: str, user: _UserModel | None = ..., password_validators: Sequence[PasswordValidator] | None = ...
) -> None: ...
def password_changed(
    password: str, user: _UserModel | None = ..., password_validators: Sequence[PasswordValidator] | None = ...
) -> None: ...
def password_validators_help_texts(password_validators: Sequence[PasswordValidator] | None = ...) -> list[str]: ...

password_validators_help_text_html: Any

class MinimumLengthValidator:
    min_length: int
    def __init__(self, min_length: int = ...) -> None: ...
    def validate(self, password: str, user: _UserModel | None = ...) -> None: ...
    def get_help_text(self) -> str: ...

class UserAttributeSimilarityValidator:
    DEFAULT_USER_ATTRIBUTES: Sequence[str]
    user_attributes: Sequence[str]
    max_similarity: float
    def __init__(self, user_attributes: Sequence[str] = ..., max_similarity: float = ...) -> None: ...
    def validate(self, password: str, user: _UserModel | None = ...) -> None: ...
    def get_help_text(self) -> str: ...

class CommonPasswordValidator:
    DEFAULT_PASSWORD_LIST_PATH: Path
    passwords: set[str]
    def __init__(self, password_list_path: Path | PosixPath | str = ...) -> None: ...
    def validate(self, password: str, user: _UserModel | None = ...) -> None: ...
    def get_help_text(self) -> str: ...

class NumericPasswordValidator:
    def validate(self, password: str, user: _UserModel | None = ...) -> None: ...
    def get_help_text(self) -> str: ...



from typing import List, Union

from django.db.models import Model, base
from pydantic import BaseModel, ConfigDict


class SeedSet(BaseModel):
    models: List[Union[Model, base.ModelBase]]
    seed_name: str

    model_config = ConfigDict(arbitrary_types_allowed=True)



from typing import Any

from django.contrib.gis.db.backends.base.features import BaseSpatialFeatures
from django.db.backends.mysql.features import DatabaseFeatures as MySQLDatabaseFeatures

class DatabaseFeatures(BaseSpatialFeatures, MySQLDatabaseFeatures):
    has_spatialrefsys_table: bool
    supports_add_srs_entry: bool
    supports_distance_geodetic: bool
    supports_length_geodetic: bool
    supports_area_geodetic: bool
    supports_transform: bool
    supports_null_geometries: bool
    supports_num_points_poly: bool
    @property
    def empty_intersection_returns_none(self) -> bool: ...
    @property
    def supports_geometry_field_unique_index(self) -> bool: ...  # type: ignore
    @property
    def django_test_skips(self) -> dict[str, Any]: ...  # type: ignore



from collections.abc import Callable
from logging import Logger
from typing import Any

from django.http.request import HttpRequest
from django.http.response import HttpResponseBase, HttpResponseForbidden
from django.utils.deprecation import MiddlewareMixin

logger: Logger
REASON_NO_REFERER: str
REASON_BAD_REFERER: str
REASON_NO_CSRF_COOKIE: str
REASON_BAD_TOKEN: str
REASON_MALFORMED_REFERER: str
REASON_INSECURE_REFERER: str
CSRF_SECRET_LENGTH: int
CSRF_TOKEN_LENGTH: Any
CSRF_ALLOWED_CHARS: Any
CSRF_SESSION_KEY: str

def get_token(request: HttpRequest) -> str: ...
def rotate_token(request: HttpRequest) -> None: ...

class CsrfViewMiddleware(MiddlewareMixin):
    def process_request(self, request: HttpRequest) -> None: ...
    def process_view(
        self, request: HttpRequest, callback: Callable | None, callback_args: tuple, callback_kwargs: dict[str, Any]
    ) -> HttpResponseForbidden | None: ...
    def process_response(self, request: HttpRequest, response: HttpResponseBase) -> HttpResponseBase: ...

def _get_new_csrf_string() -> str: ...
def _get_new_csrf_token() -> str: ...



from typing import ClassVar

from . import Image as Image, ImageFile as ImageFile
from ._binary import o8 as o8

class QoiImageFile(ImageFile.ImageFile):
    format: ClassVar[str]
    format_description: ClassVar[str]

class QoiDecoder(ImageFile.PyDecoder):
    def decode(self, buffer): ...



from collections.abc import Mapping
from typing import Any

from django.template.base import Template
from django.template.response import SimpleTemplateResponse
from django.test.utils import ContextList
from django.urls import ResolverMatch
from rest_framework.request import Request
from rest_framework.test import APIClient

class Response(SimpleTemplateResponse):
    data: Any
    exception: bool
    content_type: str | None
    _request: Request
    def __init__(
        self,
        data: Any = ...,
        status: int | None = ...,
        template_name: str | None = ...,
        headers: Mapping[str, str] | None = ...,
        exception: bool = ...,
        content_type: str | None = ...,
    ): ...
    @property
    def rendered_content(self) -> Any: ...
    def render(self) -> Any: ...
    @property
    def status_text(self) -> str: ...

class _MonkeyPatchedResponse(Response):
    client: APIClient
    context: ContextList | dict[str, Any]
    redirect_chain: list[tuple[str, int]]
    request: dict[str, Any]
    resolver_match: ResolverMatch
    templates: list[Template]
    def json(self) -> Any: ...



from typing import Any

from django.db.backends.base.introspection import BaseDatabaseIntrospection
from django.db.backends.postgresql.base import DatabaseWrapper

class DatabaseIntrospection(BaseDatabaseIntrospection):
    connection: DatabaseWrapper
    data_types_reverse: Any
    ignored_tables: Any
    def get_field_type(self, data_type: Any, description: Any) -> Any: ...
    def get_table_list(self, cursor: Any) -> Any: ...
    def get_table_description(self, cursor: Any, table_name: Any) -> Any: ...
    def get_sequences(self, cursor: Any, table_name: Any, table_fields: Any = ...) -> Any: ...
    def get_relations(self, cursor: Any, table_name: Any) -> Any: ...
    def get_key_columns(self, cursor: Any, table_name: Any) -> Any: ...
    def get_constraints(self, cursor: Any, table_name: Any) -> Any: ...



import sys
from _typeshed import ReadableBuffer
from typing import Literal, overload

if sys.platform == "win32":
    SND_APPLICATION: Literal[128]
    SND_FILENAME: Literal[131072]
    SND_ALIAS: Literal[65536]
    SND_LOOP: Literal[8]
    SND_MEMORY: Literal[4]
    SND_PURGE: Literal[64]
    SND_ASYNC: Literal[1]
    SND_NODEFAULT: Literal[2]
    SND_NOSTOP: Literal[16]
    SND_NOWAIT: Literal[8192]

    MB_ICONASTERISK: Literal[64]
    MB_ICONEXCLAMATION: Literal[48]
    MB_ICONHAND: Literal[16]
    MB_ICONQUESTION: Literal[32]
    MB_OK: Literal[0]
    def Beep(frequency: int, duration: int) -> None: ...
    # Can actually accept anything ORed with 4, and if not it's definitely str, but that's inexpressible
    @overload
    def PlaySound(sound: ReadableBuffer | None, flags: Literal[4]) -> None: ...
    @overload
    def PlaySound(sound: str | ReadableBuffer | None, flags: int) -> None: ...
    def MessageBeep(type: int = 0) -> None: ...



from datetime import date
from datetime import datetime as builtin_datetime
from datetime import time, timedelta
from re import Pattern

date_re: Pattern[str]
time_re: Pattern[str]
datetime_re: Pattern[str]
standard_duration_re: Pattern[str]
iso8601_duration_re: Pattern[str]
postgres_interval_re: Pattern[str]

def parse_date(value: str) -> date | None: ...
def parse_time(value: str) -> time | None: ...
def parse_datetime(value: str) -> builtin_datetime | None: ...
def parse_duration(value: str) -> timedelta | None: ...



from collections.abc import Callable
from typing import Any

from django.http.request import HttpRequest
from django.http.response import HttpResponse, HttpResponseBase
from django.utils.deprecation import MiddlewareMixin

class XViewMiddleware(MiddlewareMixin):
    def process_view(
        self,
        request: HttpRequest,
        view_func: Callable[..., HttpResponseBase],
        view_args: tuple,
        view_kwargs: dict[Any, Any],
    ) -> HttpResponse | None: ...



from collections.abc import Callable, Iterable, Iterator, Sequence
from datetime import date, datetime
from decimal import Decimal
from typing import Any, overload
from uuid import UUID

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.utils import CursorWrapper
from django.db.models.base import Model
from django.db.models.expressions import BaseExpression, Expression
from django.db.models.sql.query import Query
from django.db.models.sql.subqueries import AggregateQuery, DeleteQuery, InsertQuery, UpdateQuery
from typing_extensions import Literal, TypeAlias

_ParamT: TypeAlias = str | int

_ParamsT: TypeAlias = list[_ParamT]
_AsSqlType: TypeAlias = tuple[str, _ParamsT]

class SQLCompiler:
    query: Query
    connection: BaseDatabaseWrapper
    using: str | None
    quote_cache: Any
    select: Any
    annotation_col_map: Any
    klass_info: Any
    ordering_parts: Any
    def __init__(self, query: Query, connection: BaseDatabaseWrapper, using: str | None) -> None: ...
    col_count: int | None
    def setup_query(self) -> None: ...
    has_extra_select: Any
    def pre_sql_setup(
        self,
    ) -> tuple[
        list[tuple[Expression, _AsSqlType, None]],
        list[tuple[Expression, tuple[str, _ParamsT, bool]]],
        list[_AsSqlType],
    ]: ...
    def get_group_by(
        self,
        select: list[tuple[BaseExpression, _AsSqlType, str | None]],
        order_by: list[tuple[Expression, tuple[str, _ParamsT, bool]]],
    ) -> list[_AsSqlType]: ...
    def collapse_group_by(
        self, expressions: list[Expression], having: list[Expression] | tuple
    ) -> list[Expression]: ...
    def get_select(
        self,
    ) -> tuple[list[tuple[Expression, _AsSqlType, str | None]], dict[str, Any] | None, dict[str, int]]: ...
    def get_order_by(self) -> list[tuple[Expression, tuple[str, _ParamsT, bool]]]: ...
    def get_extra_select(
        self,
        order_by: list[tuple[Expression, tuple[str, _ParamsT, bool]]],
        select: list[tuple[Expression, _AsSqlType, str | None]],
    ) -> list[tuple[Expression, _AsSqlType, None]]: ...
    def quote_name_unless_alias(self, name: str) -> str: ...
    def compile(self, node: BaseExpression) -> _AsSqlType: ...
    def get_combinator_sql(self, combinator: str, all: bool) -> tuple[list[str], list[int] | list[str]]: ...
    def as_sql(self, with_limits: bool = ..., with_col_aliases: bool = ...) -> _AsSqlType: ...
    def get_default_columns(
        self, start_alias: str | None = ..., opts: Any | None = ..., from_parent: type[Model] | None = ...
    ) -> list[Expression]: ...
    def get_distinct(self) -> tuple[list[Any], list[Any]]: ...
    def find_ordering_name(
        self,
        name: str,
        opts: Any,
        alias: str | None = ...,
        default_order: str = ...,
        already_seen: set[tuple[tuple[tuple[str, str]] | None, tuple[tuple[str, str]]]] | None = ...,
    ) -> list[tuple[Expression, bool]]: ...
    def get_from_clause(self) -> tuple[list[str], _ParamsT]: ...
    def get_related_selections(
        self,
        select: list[tuple[Expression, str | None]],
        opts: Any | None = ...,
        root_alias: str | None = ...,
        cur_depth: int = ...,
        requested: dict[str, dict[str, dict[str, dict[Any, Any]]]] | None = ...,
        restricted: bool | None = ...,
    ) -> list[dict[str, Any]]: ...
    def get_select_for_update_of_arguments(self) -> list[Any]: ...
    def deferred_to_columns(self) -> dict[type[Model], set[str]]: ...
    def get_converters(self, expressions: list[Expression]) -> dict[int, tuple[list[Callable], Expression]]: ...
    def apply_converters(
        self, rows: Iterable[Iterable[Any]], converters: dict[int, tuple[list[Callable], Expression]]
    ) -> Iterator[list[None | date | datetime | float | Decimal | UUID | bytes | str]]: ...
    def results_iter(
        self,
        results: Iterable[list[Sequence[Any]]] | None = ...,
        tuple_expected: bool = ...,
        chunked_fetch: bool = ...,
        chunk_size: int = ...,
    ) -> Iterator[Sequence[Any]]: ...
    def has_results(self) -> bool: ...
    @overload
    def execute_sql(  # type: ignore
        self, result_type: Literal["cursor"] = ..., chunked_fetch: bool = ..., chunk_size: int = ...
    ) -> CursorWrapper: ...
    @overload
    def execute_sql(
        self, result_type: Literal["no results"] | None = ..., chunked_fetch: bool = ..., chunk_size: int = ...
    ) -> None: ...
    @overload
    def execute_sql(  # type: ignore
        self, result_type: Literal["single"] = ..., chunked_fetch: bool = ..., chunk_size: int = ...
    ) -> Iterable[Sequence[Any]] | None: ...
    @overload
    def execute_sql(
        self, result_type: Literal["multi"] = ..., chunked_fetch: bool = ..., chunk_size: int = ...
    ) -> Iterable[list[Sequence[Any]]] | None: ...
    def as_subquery_condition(self, alias: str, columns: list[str], compiler: SQLCompiler) -> _AsSqlType: ...
    def explain_query(self) -> Iterator[str]: ...

class SQLInsertCompiler(SQLCompiler):
    query: InsertQuery
    returning_fields: Sequence[Any] | None
    returning_params: Sequence[Any]
    def field_as_sql(self, field: Any, val: Any) -> _AsSqlType: ...
    def prepare_value(self, field: Any, value: Any) -> Any: ...
    def pre_save_val(self, field: Any, obj: Any) -> Any: ...
    def assemble_as_sql(self, fields: Any, value_rows: Any) -> tuple[list[list[str]], list[list[Any]]]: ...
    def as_sql(self) -> list[_AsSqlType]: ...  # type: ignore
    def execute_sql(  # type: ignore
        self, returning_fields: Sequence[str] | None = ...
    ) -> list[tuple[Any]]: ...  # 1-tuple

class SQLDeleteCompiler(SQLCompiler):
    query: DeleteQuery
    @property
    def single_alias(self) -> bool: ...
    @property
    def contains_self_reference_subquery(self) -> bool: ...
    def as_sql(self) -> _AsSqlType: ...  # type: ignore

class SQLUpdateCompiler(SQLCompiler):
    query: UpdateQuery
    def as_sql(self) -> _AsSqlType: ...  # type: ignore
    def execute_sql(self, result_type: Literal["cursor", "no results"]) -> int: ...  # type: ignore
    def pre_sql_setup(self) -> None: ...  # type: ignore

class SQLAggregateCompiler(SQLCompiler):
    query: AggregateQuery
    col_count: int
    def as_sql(self) -> _AsSqlType: ...  # type: ignore

def cursor_iter(
    cursor: CursorWrapper, sentinel: Any, col_count: int | None, itersize: int
) -> Iterator[list[Sequence[Any]]]: ...



from .general import ArrayAgg as ArrayAgg
from .general import BitAnd as BitAnd
from .general import BitOr as BitOr
from .general import BoolAnd as BoolAnd
from .general import BoolOr as BoolOr
from .general import JSONBAgg as JSONBAgg
from .general import StringAgg as StringAgg
from .statistics import Corr as Corr
from .statistics import CovarPop as CovarPop
from .statistics import RegrAvgX as RegrAvgX
from .statistics import RegrAvgY as RegrAvgY
from .statistics import RegrCount as RegrCount
from .statistics import RegrIntercept as RegrIntercept
from .statistics import RegrR2 as RegrR2
from .statistics import RegrSlope as RegrSlope
from .statistics import RegrSXX as RegrSXX
from .statistics import RegrSXY as RegrSXY
from .statistics import RegrSYY as RegrSYY
from .statistics import StatAggregate as StatAggregate



import sys
from _typeshed import StrOrBytesPath
from collections.abc import Callable, Sequence
from typing_extensions import TypeAlias

if sys.platform != "win32":
    _Completer: TypeAlias = Callable[[str, int], str | None]
    _CompDisp: TypeAlias = Callable[[str, Sequence[str], int], None]

    def parse_and_bind(string: str, /) -> None: ...
    def read_init_file(filename: StrOrBytesPath | None = None, /) -> None: ...
    def get_line_buffer() -> str: ...
    def insert_text(string: str, /) -> None: ...
    def redisplay() -> None: ...
    def read_history_file(filename: StrOrBytesPath | None = None, /) -> None: ...
    def write_history_file(filename: StrOrBytesPath | None = None, /) -> None: ...
    def append_history_file(nelements: int, filename: StrOrBytesPath | None = None, /) -> None: ...
    def get_history_length() -> int: ...
    def set_history_length(length: int, /) -> None: ...
    def clear_history() -> None: ...
    def get_current_history_length() -> int: ...
    def get_history_item(index: int, /) -> str: ...
    def remove_history_item(pos: int, /) -> None: ...
    def replace_history_item(pos: int, line: str, /) -> None: ...
    def add_history(string: str, /) -> None: ...
    def set_auto_history(enabled: bool, /) -> None: ...
    def set_startup_hook(function: Callable[[], object] | None = None, /) -> None: ...
    def set_pre_input_hook(function: Callable[[], object] | None = None, /) -> None: ...
    def set_completer(function: _Completer | None = None, /) -> None: ...
    def get_completer() -> _Completer | None: ...
    def get_completion_type() -> int: ...
    def get_begidx() -> int: ...
    def get_endidx() -> int: ...
    def set_completer_delims(string: str, /) -> None: ...
    def get_completer_delims() -> str: ...
    def set_completion_display_matches_hook(function: _CompDisp | None = None, /) -> None: ...



from typing import TYPE_CHECKING

from django.core.files.storage import default_storage

if TYPE_CHECKING:
    from django.core.files.storage import Storage


def file_exists_in_storage(file_path) -> bool:
    storage: "Storage" = default_storage

    return storage.exists(file_path)



from collections.abc import Sequence
from typing import Any

from django.apps.config import AppConfig
from django.core.checks.messages import Warning

def add_session_cookie_message(message: str) -> str: ...

W010: Warning
W011: Warning
W012: Warning

def add_httponly_message(message: str) -> str: ...

W013: Warning
W014: Warning
W015: Warning

def check_session_cookie_secure(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_session_cookie_httponly(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...



from collections.abc import Iterator
from typing import Any

from django.http.request import HttpRequest

class PermLookupDict:
    app_label: str
    user: Any
    def __init__(self, user: Any, app_label: str) -> None: ...
    def __getitem__(self, perm_name: str) -> bool: ...
    def __iter__(self) -> Iterator[Any]: ...
    def __bool__(self) -> bool: ...

class PermWrapper:
    user: Any
    def __init__(self, user: Any) -> None: ...
    def __getitem__(self, app_label: str) -> PermLookupDict: ...
    def __iter__(self) -> Iterator[Any]: ...
    def __contains__(self, perm_name: Any) -> bool: ...

def auth(request: HttpRequest) -> dict[str, Any]: ...



from typing import List, Type

from django.db.models import Count, Model
from django.db.models.query import QuerySet
from nwon_baseline.print_helper import print_error, print_warning


def cleanup_duplicates(dry_run: bool, fields: List[str], model: Type[Model]):
    """Cleanup duplicate model instances based in a list of fields"""

    if hasattr(model, "all_objects"):
        model_dicts = (
            model.all_objects.values(*fields)
            .annotate(Count("id"))
            .order_by()
            .filter(id__count__gt=1)
        )
    else:
        model_dicts = (
            model.objects.values(*fields)
            .annotate(Count("id"))
            .order_by()
            .filter(id__count__gt=1)
        )

    print_warning(f"Processing {model_dicts.count()} duplicates for {model.__name__}")

    for model_dict in model_dicts:
        model_dict.pop("id__count")
        model_qs: QuerySet[Model] = model.objects.filter(**model_dict)
        instance_to_keep = model_qs.last()

        if isinstance(instance_to_keep, model):
            instance_to_delete = model_qs.exclude(pk=instance_to_keep.pk)

            delete_ids = [str(instance.pk) for instance in instance_to_delete]
            delete_ids_str = ", ".join(delete_ids)

            print_warning(
                f"Keeping {model.__name__} with pk {instance_to_keep.pk} and delete "
                f"{instance_to_delete.count()} {model.__name__} with pks "
                f"{delete_ids_str} "
            )

            delete_ids = [instance.pk for instance in instance_to_delete]

            if dry_run is False:
                print_error(
                    f"Deleting {instance_to_delete.count()} {model.__name__} with pks "
                    f"{delete_ids_str} ",
                )
                instance_to_delete.delete()
            else:
                print_error(
                    f"Would delete {instance_to_delete.count()} {model.__name__} with "
                    f"pks {delete_ids_str} ",
                )


__all__ = ["cleanup_duplicates"]



import datetime
import uuid
from collections import OrderedDict
from collections.abc import Callable, Generator, Iterable, Mapping, MutableMapping, Sequence
from decimal import Decimal
from enum import Enum
from json import JSONDecoder, JSONEncoder
from re import Pattern
from typing import Any, Generic, NoReturn, Protocol, TypeVar

from _typeshed import Self
from django.core.files.base import File
from django.db import models
from django.forms import ImageField as DjangoImageField  # noqa: F401
from django_stubs_ext import StrOrPromise
from rest_framework.serializers import BaseSerializer
from rest_framework.validators import Validator
from typing_extensions import Final, TypeAlias

class _Empty(Enum):
    sentinel = 0  # noqa: Y015

empty: Final = _Empty.sentinel

class BuiltinSignatureError(Exception): ...

class CreateOnlyDefault:
    requires_context: bool
    default: Any
    def __init__(self, default: Any) -> None: ...
    def __call__(self, serializer_field: Field): ...

class CurrentUserDefault:
    requires_context: bool
    def __call__(self, serializer_field: Field): ...

class SkipField(Exception): ...

class Option(Protocol):
    start_option_group: bool
    end_option_group: bool
    label: StrOrPromise
    value: str
    display_text: StrOrPromise

def is_simple_callable(obj: Callable) -> bool: ...
def get_attribute(instance: Any, attrs: list[str] | None) -> Any: ...
def set_value(dictionary: MutableMapping[str, Any], keys: Sequence[str], value: Any) -> None: ...
def to_choices_dict(choices: Iterable[Any]) -> OrderedDict: ...
def flatten_choices_dict(choices: dict[Any, Any]) -> OrderedDict: ...
def iter_options(
    grouped_choices: OrderedDict, cutoff: int | None = ..., cutoff_text: StrOrPromise | None = ...
) -> Generator[Option, None, None]: ...
def get_error_detail(exc_info: Any) -> Any: ...

REGEX_TYPE: Pattern
NOT_READ_ONLY_WRITE_ONLY: str
NOT_READ_ONLY_REQUIRED: str
NOT_REQUIRED_DEFAULT: str
USE_READONLYFIELD: str
MISSING_ERROR_MESSAGE: str

_IN = TypeVar("_IN")  # Instance Type
_VT = TypeVar("_VT")  # Value Type
_DT = TypeVar("_DT")  # Data Type
_RP = TypeVar("_RP")  # Representation Type

class SupportsToPython(Protocol):
    def to_python(self, value: Any) -> Any: ...

_DefaultInitial: TypeAlias = _VT | Callable[[], _VT] | None | _Empty

class Field(Generic[_VT, _DT, _RP, _IN]):
    allow_null: bool
    default: _VT | None
    default_empty_html: Any
    default_error_messages: dict[str, StrOrPromise]
    default_validators: list[Validator[_VT]]
    error_messages: dict[str, StrOrPromise]
    field_name: str | None
    help_text: StrOrPromise | None
    initial: _VT | Callable[[], _VT] | None
    label: StrOrPromise | None
    parent: BaseSerializer
    read_only: bool
    required: bool
    source: Callable | str | None
    source_attrs: list[str]
    style: dict[str, Any]
    write_only: bool
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[_VT] = ...,
        initial: _DefaultInitial[_VT] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[_VT]] | None = ...,
        allow_null: bool = ...,
    ): ...
    def bind(self, field_name: str, parent: BaseSerializer) -> None: ...
    @property
    def validators(self) -> list[Validator[_VT]]: ...
    @validators.setter
    def validators(self, validators: list[Validator[_VT]]) -> None: ...
    def get_validators(self) -> list[Validator[_VT]]: ...
    def get_initial(self) -> _VT | None: ...
    def get_value(self, dictionary: Mapping[Any, Any]) -> Any: ...
    def get_attribute(self, instance: _IN) -> _RP | None: ...
    def get_default(self) -> _VT | None: ...
    def validate_empty_values(self, data: Any) -> tuple[bool, Any]: ...
    def run_validation(self, data: Any = ...) -> Any: ...
    def run_validators(self, value: Any) -> None: ...
    def to_internal_value(self, data: _DT) -> _VT: ...
    def to_representation(self, value: _VT) -> _RP: ...
    def fail(self, key: str, **kwargs: Any) -> NoReturn: ...
    @property
    def root(self) -> BaseSerializer: ...
    @property
    def context(self) -> dict[str, Any]: ...
    def __new__(cls: type[Self], *args: Any, **kwargs: Any) -> Self: ...
    def __deepcopy__(self, memo: Mapping[Any, Any]) -> Field: ...

class BooleanField(
    Field[
        bool,
        str | bool | int,
        bool,
        Any,
    ]
):
    TRUE_VALUES: set[str | bool | int]
    FALSE_VALUES: set[str | bool | int | float]
    NULL_VALUES: set[str | None]

class NullBooleanField(
    Field[
        bool | None,
        str | bool | int | None,
        bool,
        Any,
    ]
):
    TRUE_VALUES: set[str | bool | int]
    FALSE_VALUES: set[str | bool | int | float]
    NULL_VALUES: set[str | None]

class CharField(Field[str, str, str, Any]):
    allow_blank: bool
    trim_whitespace: bool
    max_length: int | None
    min_length: int | None
    def __init__(
        self,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[StrOrPromise] = ...,
        initial: _DefaultInitial[str] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[str]] | None = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        max_length: int = ...,
        min_length: int | None = ...,
    ): ...

class EmailField(CharField): ...

class RegexField(CharField):
    def __init__(
        self,
        regex: str | Pattern,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[StrOrPromise] = ...,
        initial: _DefaultInitial[str] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[str]] | None = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        max_length: int = ...,
        min_length: int | None = ...,
    ): ...

class SlugField(CharField):
    allow_unicode: bool
    def __init__(
        self,
        allow_unicode: bool = ...,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[StrOrPromise] = ...,
        initial: _DefaultInitial[str] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[str]] | None = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        max_length: int = ...,
        min_length: int | None = ...,
    ): ...

class URLField(CharField): ...

class UUIDField(Field[uuid.UUID, uuid.UUID | str | int, str, Any]):
    valid_formats: Sequence[str]
    uuid_format: str
    def __init__(
        self,
        *,
        format: str | None = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[uuid.UUID] = ...,
        initial: _DefaultInitial[uuid.UUID] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[uuid.UUID]] | None = ...,
        allow_null: bool = ...,
    ): ...

class IPAddressField(CharField):
    protocol: str
    unpack_ipv4: bool
    def __init__(
        self,
        protocol: str = ...,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[str] = ...,
        initial: _DefaultInitial[str] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[str]] | None = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        max_length: int = ...,
        min_length: int | None = ...,
    ): ...

class IntegerField(Field[int, float | int | str, int, Any]):
    MAX_STRING_LENGTH: int
    re_decimal: Pattern
    max_value: int | None
    min_value: int | None
    def __init__(
        self,
        *,
        max_value: int = ...,
        min_value: int = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[int] = ...,
        initial: _DefaultInitial[int] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[int]] | None = ...,
        allow_null: bool = ...,
    ): ...

class FloatField(Field[float, float | int | str, str, Any]):
    MAX_STRING_LENGTH: int
    re_decimal: Pattern
    max_value: float | None
    min_value: float | None
    def __init__(
        self,
        *,
        max_value: float = ...,
        min_value: float = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[float] = ...,
        initial: _DefaultInitial[float] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[float]] | None = ...,
        allow_null: bool = ...,
    ): ...

class DecimalField(Field[Decimal, int | float | str | Decimal, str, Any]):
    MAX_STRING_LENGTH: int
    max_digits: int | None
    decimal_places: int | None
    coerce_to_string: bool | None
    max_value: Decimal | int | float | None
    min_value: Decimal | int | float | None
    localize: bool
    rounding: str | None
    max_whole_digits = int | None  # noqa: Y026
    def __init__(
        self,
        max_digits: int | None,
        decimal_places: int | None,
        coerce_to_string: bool = ...,
        max_value: Decimal | int | float = ...,
        min_value: Decimal | int | float = ...,
        localize: bool = ...,
        rounding: str | None = ...,
        normalize_output: bool = ...,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[Decimal] = ...,
        initial: _DefaultInitial[Decimal] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[Decimal]] | None = ...,
        allow_null: bool = ...,
    ): ...
    def validate_precision(self, value: Decimal) -> Decimal: ...
    def quantize(self, value: Decimal) -> Decimal: ...

class DateTimeField(Field[datetime.datetime, datetime.datetime | str, str, Any]):
    datetime_parser: Callable[[str, str], datetime.datetime]
    format: str | None
    input_formats: Sequence[str]
    timezone: datetime.tzinfo
    def __init__(
        self,
        format: str | None = ...,
        input_formats: Sequence[str] = ...,
        default_timezone: datetime.tzinfo | None = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[datetime.datetime] = ...,
        initial: _DefaultInitial[datetime.datetime] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[datetime.datetime]] | None = ...,
        allow_null: bool = ...,
    ): ...
    def enforce_timezone(self, value: datetime.datetime) -> datetime.datetime: ...
    def default_timezone(self) -> str | None: ...

class DateField(Field[datetime.date, datetime.date | str, str, Any]):
    datetime_parser: Callable[[str, str], datetime.datetime]
    format: str | None
    input_formats: Sequence[str]
    def __init__(
        self,
        format: str | None = ...,
        input_formats: Sequence[str] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[datetime.date] = ...,
        initial: _DefaultInitial[datetime.date] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[datetime.date]] | None = ...,
        allow_null: bool = ...,
    ): ...

class TimeField(Field[datetime.time, datetime.time | str, str, Any]):
    datetime_parser: Callable[[str, str], datetime.datetime]
    format: str | None
    input_formats: Sequence[str]
    def __init__(
        self,
        format: str | None = ...,
        input_formats: Sequence[str] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[datetime.time] = ...,
        initial: _DefaultInitial[datetime.time] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[datetime.time]] | None = ...,
        allow_null: bool = ...,
    ): ...

class DurationField(Field[datetime.timedelta, datetime.timedelta | str, str, Any]):
    max_value: datetime.timedelta | None
    min_value: datetime.timedelta | None
    def __init__(
        self,
        *,
        max_value: datetime.timedelta | int | float = ...,
        min_value: datetime.timedelta | int | float = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[datetime.timedelta] = ...,
        initial: _DefaultInitial[datetime.timedelta] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[datetime.timedelta]] | None = ...,
        allow_null: bool = ...,
    ): ...

class ChoiceField(Field[str, str | int | tuple[str | int, str | int | tuple], str, Any]):
    html_cutoff: int | None
    html_cutoff_text: StrOrPromise | None
    allow_blank: bool
    grouped_choices: OrderedDict
    choice_strings_to_values: dict[str, Any]
    _choices: OrderedDict
    def __init__(
        self,
        choices: Iterable[Any],
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[StrOrPromise | int] = ...,
        initial: _DefaultInitial[str | int] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[Any]] | None = ...,
        allow_null: bool = ...,
        html_cutoff: int = ...,
        html_cutoff_text: StrOrPromise = ...,
        allow_blank: bool = ...,
    ): ...
    def iter_options(self) -> Iterable[Option]: ...
    def _get_choices(self) -> dict[Any, Any]: ...
    def _set_choices(self, choices: Iterable[Any]) -> None: ...
    choices = property(_get_choices, _set_choices)

class MultipleChoiceField(
    ChoiceField,
    Field[
        str,
        Sequence[str | int | tuple[str | int, str | int]],
        Sequence[str | tuple[str | int, str | int]],
        Any,
    ],
):
    allow_empty: bool
    def __init__(
        self,
        choices: Iterable[Any],
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[set[str | int] | set[str] | set[int]] = ...,
        initial: _DefaultInitial[set[StrOrPromise | int] | set[StrOrPromise] | set[int]] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[Any]] | None = ...,
        allow_null: bool = ...,
        html_cutoff: int = ...,
        html_cutoff_text: StrOrPromise = ...,
        allow_blank: bool = ...,
        allow_empty: bool = ...,
    ): ...

class FilePathField(ChoiceField):
    def __init__(
        self,
        path: str,
        match: str = ...,
        recursive: bool = ...,
        allow_files: bool = ...,
        allow_folders: bool = ...,
        required: bool = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        default: _DefaultInitial[str] = ...,
        initial: _DefaultInitial[str] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[Any]] | None = ...,
        allow_null: bool = ...,
        html_cutoff: int = ...,
        html_cutoff_text: StrOrPromise = ...,
        allow_blank: bool = ...,
    ): ...

class FileField(Field[File, File, str | None, Any]):  # this field can return None without raising!
    max_length: int
    allow_empty_file: bool
    use_url: bool
    def __init__(
        self,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[File] = ...,
        initial: _DefaultInitial[File] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[File]] | None = ...,
        allow_null: bool = ...,
        max_length: int = ...,
        allow_empty_file: bool = ...,
        use_url: bool = ...,
    ): ...

class ImageField(FileField):
    _DjangoImageField: SupportsToPython
    def __init__(
        self,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[File] = ...,
        initial: _DefaultInitial[File] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[File]] | None = ...,
        allow_null: bool = ...,
        max_length: int = ...,
        allow_empty_file: bool = ...,
        use_url: bool = ...,
        _DjangoImageField: type[SupportsToPython] = ...,
    ): ...

class _UnvalidatedField(Field): ...

class ListField(Field[list[Any], list[Any], list[Any], Any]):
    child: Field
    allow_empty: bool
    max_length: int | None
    min_length: int | None
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[list[Any]] = ...,
        initial: _DefaultInitial[list[Any]] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[list[Any]]] | None = ...,
        allow_null: bool = ...,
        *,
        child: Field = ...,
        allow_empty: bool = ...,
        max_length: int = ...,
        min_length: int = ...,
    ): ...
    def run_child_validation(self, data: list[Mapping[Any, Any]]) -> Any: ...

class DictField(Field[dict[Any, Any], dict[Any, Any], dict[Any, Any], Any]):
    child: Field
    allow_empty: bool
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[dict[Any, Any]] = ...,
        initial: _DefaultInitial[dict[Any, Any]] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[dict[Any, Any]]] | None = ...,
        allow_null: bool = ...,
        *,
        child: Field = ...,
        allow_empty: bool = ...,
    ): ...
    def run_child_validation(self, data: Any) -> Any: ...

class HStoreField(DictField):
    child: CharField

class JSONField(Field[dict[str, Any] | list[dict[str, Any]], dict[str, Any] | list[dict[str, Any]], str, Any]):
    binary: bool
    encoder: type[JSONEncoder] | None
    decoder: type[JSONDecoder] | None
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[Mapping[Any, Any]] = ...,
        initial: _DefaultInitial[Mapping[Any, Any]] = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[Any]] | None = ...,
        allow_null: bool = ...,
        *,
        binary: bool = ...,
        encoder: type[JSONEncoder] | None = ...,
        decoder: type[JSONDecoder] | None = ...,
    ): ...

class ReadOnlyField(Field): ...
class HiddenField(Field): ...

class SerializerMethodField(Field):
    method_name: str
    def __init__(
        self,
        method_name: str | None = ...,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[Any]] | None = ...,
        allow_null: bool = ...,
    ): ...

class ModelField(Field):
    model_field: models.Field
    max_length: int
    def __init__(
        self,
        model_field: models.Field,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: str = ...,
        label: StrOrPromise = ...,
        help_text: StrOrPromise = ...,
        style: dict[str, Any] = ...,
        error_messages: dict[str, StrOrPromise] = ...,
        validators: Sequence[Validator[Any]] | None = ...,
        allow_null: bool = ...,
        max_length: int = ...,
    ): ...
